/**************************************************************************//**
 * @file     system_A33M11x.c
 * @brief    CMSIS Cortex-M3 Device Peripheral Access Layer Source File for
 *           Device A33M11x
 * @version  V1.00
 * @date       2020
 *
 * @note
 *
 ******************************************************************************/
/* Copyright (c) 2012 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/


#include <stdint.h>
#include "A33M11x.h"
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_cfmc.h"


/*----------------------------------------------------------------------------
  DEFINES
 *----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Define clocks
 *----------------------------------------------------------------------------*/
 
/*----------------------------------------------------------------------------
  Clock Variable definitions
 *----------------------------------------------------------------------------*/
uint32_t SystemCoreClock;     /*!< System Clock Frequency (Core Clock)  HLCK */
uint32_t SystemPeriClock;     /*!< System Clock Frequency (Peri Clock)  PCLK */


/*----------------------------------------------------------------------------
  Clock functions
 *----------------------------------------------------------------------------*/

void SystemInit (void)
{
	__disable_irq();
	
	// WDT Disable
	WDT->AEN = 0xA55A;
	WDT->CON = 0;
	WDT->AEN = 0;
	
	SystemCoreClock = 500000;				// 500KHz
	SystemPeriClock = 500000;				// 500KHz
	
	//=========================================================================
	// Flash Memory Control
	//=========================================================================
	
	// Flash Write Enable 
	HAL_CFMC_WriteCmd(ENABLE);
	
	// Code & Instruction Cache Enable
	HAL_CFMC_CacheCmd(ENABLE);

	HAL_CFMC_WaitCmd(7);		// Flash Wait 7
	
}
