/**********************************************************************//**
* @file		A33M11x_hal_afe.h
* @brief	Contains all macro definitions and function prototypes
* 			support for AFE firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_AFE_H_
#define _A33M11x_AFE_H_

/* Includes ------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif


/* Private Macros ------------------------------------------------------------- */

/* --------------------- BIT DEFINITIONS -------------------------------------- */
/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt Control register
 **********************************************************************/
	
#define AFE_CMPCR_INTTYPE0		    	((uint32_t)(0<<0))   
#define AFE_CMPCR_INTTYPE0_BITMASK    	((uint32_t)(3<<0))   
#define AFE_CMPCR_INTTYPE1		    	((uint32_t)(0<<4))  
#define AFE_CMPCR_INTTYPE1_BITMASK    	((uint32_t)(3<<4))   	
#define AFE_CMPCR_INTTYPE2		    	((uint32_t)(0<<8))
#define AFE_CMPCR_INTTYPE2_BITMASK    	((uint32_t)(3<<8))	
#define AFE_CMPCR_INTTYPE3		    	((uint32_t)(0<<12))
#define AFE_CMPCR_INTTYPE3_BITMASK    	((uint32_t)(3<<12))
	
#define AFE_CMPCR_INTTYPE_BITMASK		((uint32_t)(0x3333))

#define AFE_CMPCR_INTPOL0    				((uint32_t)(0<<3))  
#define AFE_CMPCR_INTPOL0_BITMASK  		((uint32_t)(1<<3)) 
#define AFE_CMPCR_INTPOL1    				((uint32_t)(0<<7))
#define AFE_CMPCR_INTPOL1_BITMASK  		((uint32_t)(1<<7)) 
#define AFE_CMPCR_INTPOL2    				((uint32_t)(0<<11))   
#define AFE_CMPCR_INTPOL2_BITMASK  		((uint32_t)(1<<11)) 
#define AFE_CMPCR_INTPOL3    				((uint32_t)(0<<15))
#define AFE_CMPCR_INTPOL3_BITMASK  		((uint32_t)(1<<15)) 

#define AFE_CMPCR_INTPOL_BITMASK    		((uint32_t)(0x8888))

/**********************************************************************
 * Macro defines for Macro defines for Comparator debounce register
 **********************************************************************/	
 
#define AFE_CMPDBR_C3FLTSEL			((uint32_t)(0<<12uL))
#define AFE_CMPDBR_C2FLTSEL			((uint32_t)(0<<8uL))
#define AFE_CMPDBR_C1FLTSEL			((uint32_t)(0<<4uL))
#define AFE_CMPDBR_C0FLTSEL			((uint32_t)(0<<0uL))

#define AFE_CMPDBR_FLTSEL_BITMASK	((uint32_t)(0xFFFF))

/**********************************************************************
 * Macro defines for Macro defines for Comparator status register
 **********************************************************************/
#define AFE_CMPSR_CMP0INTF			((uint32_t)(1<<1))
#define AFE_CMPSR_CMP1INTF			((uint32_t)(1<<5))
#define AFE_CMPSR_CMP2INTF			((uint32_t)(1<<9))
#define AFE_CMPSR_CMP3INTF			((uint32_t)(1<<13))

#define AFE_CMPSR_CMPINTF_BITMASK			((uint32_t)(0x1111))

#define AFE_CMPSR_CMP0FLAG			((uint32_t)(1<<0))
#define AFE_CMPSR_CMP1FLAG			((uint32_t)(1<<4))
#define AFE_CMPSR_CMP2FLAG			((uint32_t)(1<<8))
#define AFE_CMPSR_CMP3FLAG			((uint32_t)(1<<12))

/**********************************************************************
 * Macro defines for Macro defines for Analog Front End control register
 **********************************************************************/
#define AFE_CR_AMPISEL	   		((uint32_t)(0<<16))  
#define AFE_CR_AMPISEL_MASK	   		((uint32_t)(3<<16))  

#define AFE_CR_CMPOSEL	   		((uint32_t)(0<<15))  
#define AFE_CR_CMPOSEL_MASK	   		((uint32_t)(1<<15))  

#define AFE_CR_OPADISEN				((uint32_t)(0<<3))
#define AFE_CR_OPAEN					((uint32_t)(1<<3))
#define AFE_CR_OPAEN_MASK					((uint32_t)(1<<3))

#define AFE_CR_CMPIDISEN					((uint32_t)(0<<2))
#define AFE_CR_CMPIEN					((uint32_t)(1<<2))
#define AFE_CR_CMPIEN_MASK				((uint32_t)(1<<2))

#define AFE_CR_UGAINDISEN					((uint32_t)(0<<1))
#define AFE_CR_UGAINEN					((uint32_t)(1<<1))
#define AFE_CR_UGAINEN_MASK				((uint32_t)(1<<1))

#define AFE_CR_CMPDISEN			((uint32_t)(0<<0))
#define AFE_CR_CMPEN				((uint32_t)(1<<0))
#define AFE_CR_CMPEN_MASK			((uint32_t)(1<<0))


/*
 * @brief 	Comparator Interrupt mode enumerate definition
 */
enum {
	INTTYPE_NOTUSE = 0,
	INTTYPE_LEVEL,
	INTTYPE_EDGE,
	INTTYPE_BOTH_EDGE
};

enum {
	INTPOL_LOW = 0,
	INTPOL_HIGH
};

/*
 * @brief 	Comparator output polarity setting definition
*/
enum {
	OUTPOL_NONE = 0,
	OUTPOL_REVERSE
};

/*
 * @brief 	Opamp current setting definition
*/
enum {
	AMPISEL_0PERCENT = 0,
	AMPISEL_17PERCENT,
	AMPISEL_21PERCENT,
	AMPISEL_12PERCENT
};

/* Public Types --------------------------------------------------------------- */
/***********************************************************************
 * @brief Comparator enumeration
**********************************************************************/

/* Public Functions ----------------------------------------------------------- */
/* Comparator interrupt status get/clear functions --------------------------------------------------*/
uint32_t HAL_AFE_COMP_GetStatus(void);
void HAL_AFE_COMP_ClearStatus(uint32_t Type);
void HAL_AFE_COMP_ClearStatusAll(void);

/* Comparator configure functions -------------------------------------------------*/
void HAL_AFE_ClockInit(FunctionalState NewState);
HAL_Status_Type HAL_AFE_COMP_DebounceFilter(uint8_t afenum, uint32_t dbnccounter);
HAL_Status_Type HAL_AFE_COMP_Cmd(uint8_t afenum,  FunctionalState NewState);
HAL_Status_Type HAL_AFE_COMP_IntCmd(uint8_t afenum, FunctionalState NewState);
HAL_Status_Type HAL_AFE_COMP_IntConfigure(uint8_t afenum, uint32_t ipol, uint32_t mode);
HAL_Status_Type HAL_AFE_COMP_OutputConfigure(uint8_t afenum, uint32_t outpol);

/* Opamp configure functions -------------------------------------------------*/
HAL_Status_Type HAL_AFE_OPAMP_Cmd(uint8_t afenum, FunctionalState NewState);
HAL_Status_Type HAL_AFE_OPAMP_CurrentSet(uint8_t afenum,  uint8_t ampisel);
HAL_Status_Type HAL_AFE_OPAMP_UGAIN_Cmd(uint8_t afenum, FunctionalState NewState);
void HAL_AFE_ClockInit(FunctionalState NewState);

#ifdef __cplusplus
}
#endif


#endif /* _AFE_H_ */

/* --------------------------------- End Of File ------------------------------ */
