/**********************************************************************//**
* @file		A33M11x_hal_crc.h
* @brief	Contains all macro definitions and function prototypes
* 			support for CRC firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_CRC_H_
#define _A33M11x_CRC_H_

/* Includes ------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

#define CRC_OUT_INV_DIS                  (0x0uL<<21)
#define CRC_OUT_INV_EN                   (0x1uL<<21)

#define CRC_OUT_REV_DIS                  (0x0uL<<20)
#define CRC_OUT_REV_EN                   (0x1uL<<20)

#define CRC_IN_REV_DIS                   (0x0uL<<16)
#define CRC_IN_REV_EN                    (0x1uL<<16)
	
#define CRC_DMAD_INT_DIS                 (0x0uL<<8)
#define CRC_DMAD_INT_EN                  (0x1uL<<8)
	
#define CRC_INIT_DIS                     (0x0uL<<0)
#define CRC_INIT_EN                      (0x1uL<<0)

#define CRC_DMAD_INT                     (0x1uL<<8)

	
#ifdef __cplusplus
}
#endif 

typedef enum {
	CRC_POLY_CRC32 =0,
	CRC_POLY_CRC16,
	CRC_POLY_CRC8,
	CRC_POLY_CRC7
}eCRC_POLY;

/**
 * @brief CRC configuration structure definitions
 */
typedef struct {
  uint32_t Out_inv;
  uint32_t Out_rev;
  uint32_t In_rev;
  uint32_t DMAD_Int;
  uint32_t Poly;
} CRC_CFG_Type;



HAL_Status_Type HAL_CRC_Init(CRC_CFG_Type *CRC_ConfigStruct);
HAL_Status_Type HAL_CRC_SetInitVal(uint32_t init);
HAL_Status_Type HAL_CRC_ApplyInitVal(FunctionalState en);
uint32_t	HAL_CRC_GetInitVal(void);
HAL_Status_Type HAL_CRC_SetInputData8(uint8_t data8);
HAL_Status_Type HAL_CRC_SetInputData32(uint32_t data32);
uint32_t	HAL_CRC_GetOutputData(void);
uint32_t	HAL_CRC_GetStatus(void);
HAL_Status_Type HAL_CRC_ClearPending(uint32_t Type);

#endif /* _A33M11x_CRC_H_ */

/* --------------------------------- End Of File ------------------------------ */
