/**********************************************************************//**
* @file		A33M11x_hal_dma.h
* @brief	Contains all macro definitions and function prototypes
* 			support for DMAC firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_DMA_H_
#define _A33M11x_DMA_H_

/* Includes ------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif
/**
 * @brief Dma configuration structure definitions
 */
typedef struct
{
	uint32_t	transcnt;
	uint32_t	perisel;
	uint32_t	bussize;
	uint32_t	dirsel;
	uint32_t	periaddr;
	uint32_t	memaddr;
} DMA_CFG_Type;

extern enum
{
	CHANNEL_IDLE = 0,
	UART0_RX,
	UART0_TX,
	UART1_RX,
	UART1_TX,
	UART2_RX,
	UART2_TX,
	UART3_RX,
	UART3_TX,
	SPI0_RX,
	SPI0_TX,
	SPI1_RX,
	SPI1_TX,
	ADC0_RX,
	ADC1_RX,
	CRC_TX
} DMA_PERISEL;



/* Public Macros -------------------------------------------------------------- */
/*********************************************************************//**
 * DMA Control	defines
 **********************************************************************/
#define DMA_CR_BYTE_TRANS			(0<<2)
#define DMA_CR_HWORD_TRANS			(1<<2)
#define DMA_CR_WORD_TRANS			(2<<2)

#define DMA_CR_DIR_MEM_TO_PERI		(0<<1)
#define DMA_CR_DIR_PERI_TO_MEM		(1<<1)


/*********************************************************************//**
 * DMA Control	defines
 **********************************************************************/
#define DMA_SR_EOT					(1<<7)



/* Public Functions ----------------------------------------------------------- */
HAL_Status_Type HAL_DMAC_Init(void);
HAL_Status_Type HAL_DMAC_DeInit(void);
HAL_Status_Type HAL_DMAC_Setup(DMA_Type *DMAx, DMA_CFG_Type *DMACChConfig);
HAL_Status_Type HAL_DMAC_ChCmd(DMA_Type *DMAx, FunctionalState NewState);
FlagStatus HAL_DMAC_GetStatus(DMA_Type* DMAx);

#ifdef __cplusplus
}
#endif


#endif /* _A33M11x_DMA_H_ */

/* --------------------------------- End Of File ------------------------------ */
