/*************************************************************************************//**
* @file		A33M11x_hal_frt.h
* @brief	Contains all macro definitions and function prototypes
* 			support for FRT firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_FRT_H_
#define _A33M11x_FRT_H_

/* Includes ----------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @brief FRT CTRL
 */
#define	FRT_CTRL_OVFIE			(1<<9)			/*!< FRT Overflow Interrupt Enable bit */	
#define	FRT_CTRL_MATCHIE		(1<<8)			/*!< FRT Match Interrupt Enable bit */

#define FRT_CTRL_MODE_FREE_RUN	(0<<1)
#define FRT_CTRL_MODE_MATCH		(1<<1)

#define FRT_CTRL_FRT_ON			(1<<0)


/**
 * @brief FRT Flags
 */
#define	FRT_STAT_OVFI			(1<<9)			/*!< FRT Overflow Interrupt flag bit */	
#define	FRT_STAT_MATCHI			(1<<8)			/*!< FRT Match Interrupt flag bit */

/* Public Functions ----------------------------------------------------------- */
HAL_Status_Type HAL_FRT_Init(uint8_t MODE_v);

void HAL_FRT_ClockSource(uint8_t ClkSource, uint8_t ClkDivider);
uint8_t HAL_FRT_GetClockSource(void);

void HAL_FRT_Run(uint8_t isset);

void HAL_FRT_MatchInterrupt(uint8_t isset);
void HAL_FRT_OverflowInterrupt(uint8_t isset);

void HAL_FRT_ClearStatus(uint16_t StatusType);
uint16_t HAL_FRT_GetStatus(void);

uint32_t HAL_FRT_GetCounterVal(void);
void HAL_FRT_ClearCounter(void);

void HAL_FRT_SetMatchCounter(uint32_t MCNT_v);
uint32_t HAL_FRT_GetMatchCounter(void);
void HAL_FRT_ModeSelection(uint8_t MODE_v);


#ifdef __cplusplus
}
#endif


#endif /* _A33M11x_FRT_H_ */

/* --------------------------------- End Of File ------------------------------ */

