/**********************************************************************//**
* @file		A33M11x_hal_mpwm.h
* @brief	Contains all macro definitions and function prototypes
* 			support for MPWM firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_MPWM_H_
#define _A33M11x_MPWM_H_

/* Includes ------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

	

typedef enum {
	PH_U = 0,
	PH_V,
	PH_W
} MPWM_PH_VAL;


typedef enum {
	PROTPOL_LOW_ACTIVE = 0,
	PROTPOL_HIGH_ACTIVE
} MPWM_PROTPOL;


typedef enum {
	PROTSEL_PROT = 0,
	PROTSEL_OVV
} MPWM_PROTSEL;

typedef enum {
	OUTSTAT_STOP = 0,
	OUTSTAT_START_INVERT
} MPWM_OUTSTAT;

enum {
	LOW_LEVEL =0,
	HIGH_LEVEL =1
} ;

enum {
	BASIC_OUTPUT = 0,
	INVERT_OUTPUT =1
};

enum {
	LOW_SIDE = 0,
	HIGH_SIDE = 1,
	BOTH_SIDE = 2
};

typedef enum {
	MPWM_MAIN_CAP_CNT,
	MPWM_RISING_CAP_VAL,
	MPWM_FALLING_CAP_VAL,
	MPWM_SUB_CAP_VAL
} MPWM_CAP_MODE;

typedef enum {
	TRG_PRD_MAT,
	TRG_PWM_CLOCK,
} MPWM_ADCTRG_TIME;

typedef enum {
	TRG_DISABLE,
	TRG_UP_COUNT,
	TRG_DOWN_COUNT,
	TRG_UP_DOWN_COUNT
} MPWM_ADCTRG_MODE;

typedef enum {
	MPWM_Ext_STOP = 0,
	MPWM_Ext_START
} MPWM_Ext_OPERATEVAL;


typedef enum {
	MPWM_Ext_HALT = 0,
	MPWM_Ext_CONT
} MPWM_Ext_PAUSE;


typedef enum {
	RISING_CAP_VAL = 0,
	FALLING_CAP_VAL,
	SUB_CAP_VAL
} MPWM_CAP_SEL;


typedef enum {
	SUBCAP_FALLING = 0,
	SUBCAP_RISING
} MPWM_SCAP_EDGE;



/* --------------------- BIT DEFINITIONS -------------------------------------- */
/**********************************************************************//**
 * Macro defines for Macro defines for MPWM interrupt enable register
 **********************************************************************/
#define MPWM_IER_PRDWIE		((uint32_t)(1<<11)) 	/*!< Phase-W period interrupt enable*/
#define MPWM_IER_BOTWIE		((uint32_t)(1<<10)) 	/*!< Phase-W bottom interrupt enable*/
#define MPWM_IER_PRDVIE		((uint32_t)(1<<9)) 	/*!< Phase-V period interrupt enable*/
#define MPWM_IER_BOTVIE		((uint32_t)(1<<8)) 	/*!< Phase-V bottom interrupt enable */
#define MPWM_IER_PRDUIE		((uint32_t)(1<<7)) 	/*!< Phase-U period interrupt enable */
#define MPWM_IER_BOTUIE		((uint32_t)(1<<6)) 	/*!< Phase-U bottom interrupt enable */
#define MPWM_IER_WHIE_ATR6IE		((uint32_t)(1<<5)) 	/*!< WH duty or ATR6 match Interrupt enable*/
#define MPWM_IER_VHIE_ATR5IE		((uint32_t)(1<<4)) 	/*!< VH duty or ATR5 match Interrupt enable*/
#define MPWM_IER_UHIE_ATR4IE	((uint32_t)(1<<3)) 	/*!< UH duty or ATR4 match Interrupt enable*/
#define MPWM_IER_WLIE_ATR3IE		((uint32_t)(1<<2)) 	/*!< WL duty or ATR3 match Interrupt enable */
#define MPWM_IER_VLIE_ATR2IE		((uint32_t)(1<<1)) 	/*!< VL duty or ATR2match Interrupt enable */
#define MPWM_IER_ULIE_ATR1IE		((uint32_t)(1<<0)) 	/*!< UL duty or ATR1 match Interrupt enable */


/**********************************************************************//**
 * Macro defines for Macro defines for MPWM  status register
 **********************************************************************/
#define MPWM_SR_WDOWN		((uint32_t)(1<<23)) 	/*!< Phase W's Current PWM counter mode */
#define MPWM_SR_WIRQCNT		((uint32_t)(1<<20)) 	/*!< All channels or phase W channels period match interrupt count value */
#define MPWM_SR_VDOWN		((uint32_t)(1<<19)) 	/*!< Phase V's Current PWM counter mode */
#define MPWM_SR_VIRQCNT		((uint32_t)(1<<16)) 	/*!< All channels or phase V channels period match interrupt count value */
#define MPWM_SR_UDOWN		((uint32_t)(1<<15)) 	/*!< Phase U's Current PWM counter mode */
#define MPWM_SR_UIRQCNT		((uint32_t)(1<<12)) 	/*!< All channels or phase U channels period match interrupt count value */
#define MPWM_SR_PRDWIF		((uint32_t)(1<<11)) 	/*!<  Phase W's PWM period interrupt flag */
#define MPWM_SR_BOTWIF		((uint32_t)(1<<10)) 	/*!<  Phase W's PWM bottom interrupt flag */
#define MPWM_SR_PRDVIF		((uint32_t)(1<<9)) 	/*!<  Phase V's PWM period interrupt flag */
#define MPWM_SR_BOTVIF		((uint32_t)(1<<8)) 	/*!<  Phase V's PWM bottom interrupt flag */
#define MPWM_SR_PRDUIF		((uint32_t)(1<<7)) 	/*!<  Phase U's PWM period interrupt flag */
#define MPWM_SR_BOTUIF		((uint32_t)(1<<6)) 	/*!< Phase U's PWM bottom interrupt flag */
#define MPWM_SR_DWHIF_ATR6F		((uint32_t)(1<<5)) 	/*!< PWM duty WH interrupt flag */
#define MPWM_SR_DVHIF_ATF5F		((uint32_t)(1<<4)) 	/*!< PWM duty VH interrupt flag */
#define MPWM_SR_DUHIF_ATR4F		((uint32_t)(1<<3)) 	/*!< PWM duty UH interrupt flag */
#define MPWM_SR_DWLIF_ATF3F		((uint32_t)(1<<2)) 	/*!< PWM duty WL interrupt flag */
#define MPWM_SR_DVLIF_ATR2F		((uint32_t)(1<<1)) 	/*!< PWM duty VL interrupt flag */
#define MPWM_SR_DULIF_ATF1F		((uint32_t)(1<<0)) 	/*!< PWM duty UL interrupt flag */


/**********************************************************************//**
 * Macro defines for Macro defines for MPWM protection status register
 **********************************************************************/
#define MPWM_PSR_PROTKEY		((uint32_t)(0xCA<<8)) 	/*!< Protection clear access key */
#define MPWM_PSR_PROTIF		((uint32_t)(1<<7)) 	/*!< Protection interrupt status */
#define MPWM_PSR_WHPROT		((uint32_t)(1<<5)) 	/*!< Phase W's H-side protection flag */
#define MPWM_PSR_VHPROT		((uint32_t)(1<<4)) 	/*!< Phase V's H-side protection flag */
#define MPWM_PSR_UHPROT		((uint32_t)(1<<3)) 	/*!< Phase U's H-side protection flag */
#define MPWM_PSR_WLPROT		((uint32_t)(1<<2)) 	/*!< Phase W's L-side protection flag */
#define MPWM_PSR_VLPROT		((uint32_t)(1<<1)) 	/*!< Phase V's L-side protection flag */
#define MPWM_PSR_ULPROT		((uint32_t)(1<<0)) 	/*!< Phase U's L-side protection flag */


/**********************************************************************//**
 * Macro defines for Macro defines for MPWM overvoltage status register
 **********************************************************************/
#define MPWM_OSR_PROTKEY		((uint32_t)(0xAC<<8)) 	/*!< Overvoltage clear access key */
#define MPWM_OSR_OVINIF		((uint32_t)(1<<7)) 	/*!< Overvoltage interrupt status */
#define MPWM_OSR_WHOVINF		((uint32_t)(1<<5)) 	/*!< Phase W's H-side overvoltage flag */
#define MPWM_OSR_VHOVINF	((uint32_t)(1<<4)) 	/*!< Phase V's H-side overvoltage flag */
#define MPWM_OSR_UHOVINF		((uint32_t)(1<<3)) 	/*!< Phase U's H-side overvoltage flag */
#define MPWM_OSR_WLOVINF		((uint32_t)(1<<2)) 	/*!< Phase W's L-side overvoltage flag */
#define MPWM_OSR_VLOVINF		((uint32_t)(1<<1)) 	/*!< Phase V's L-side overvoltage flag */
#define MPWM_OSR_ULOVINF		((uint32_t)(1<<0)) 	/*!< Phase U's L-side overvoltage flag */


/**********************************************************************//**
 * Macro defines for MPWM Dead time setting 
 **********************************************************************/
#define PWMCLK_2		0
#define PWMCLK_4		1
#define PWMCLK_8		2
#define PWMCLK_16		3

#define PSHRT_EN		0
#define PSHRT_DIS		1

#define DTIME_DIS		0
#define DTIME_EN		1

#define LEAD_EDGE_PWMxH		0
#define TRAILL_EDGE_PWMxH		1


/**********************************************************************//**
 * Macro defines for MPWM Mode setting 
 **********************************************************************/
#define MOTOR_PWM		0
#define NORMAL_PWM		1
#define INDIVIDUAL_PWM		3

#define PRD_MAT		0
#define PWM_CLOCK		1

#define UPDATE_DISABLE		0
#define UPDATE_PERIOD		1
#define UPDATE_BOTTOM		1

#define SYMMETRIC_2CH		0
#define ASYMMETRIC_1CH		1
#define SYMMETRIC_1CH		2

#define UP_COUNT		0
#define UP_DOWN_COUNT		1

/* Public Functions ----------------------------------------------------------- */
/* MPWM Init/DeInit functions --------------------------------------------------*/
HAL_Status_Type HAL_MPWM_Init(MPWM_Type *MPWMx);
HAL_Status_Type HAL_MPWM_DeInit(MPWM_Type* MPWMx);

HAL_Status_Type HAL_MPWM_Start(MPWM_Type* MPWMx, FunctionalState NewState);
HAL_Status_Type HAL_MPWM_Halt(MPWM_Type* MPWMx, FunctionalState NewState);

HAL_Status_Type HAL_MPWM_SetOutput(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, MPWM_OUTSTAT outstatus, uint8_t side_val, uint8_t out_val);
HAL_Status_Type HAL_MPWM_SetFOutput(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint8_t side_val, uint8_t fout_val);


HAL_Status_Type HAL_MPWM_SetProtCtrl(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, MPWM_PH_VAL pha_val, MPWM_PROTPOL pol, uint32_t dbc_val, FunctionalState NewState);
HAL_Status_Type HAL_MPWM_SetProtOutput(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, MPWM_PH_VAL pha_val, uint8_t side_val, FunctionalState NewState);
HAL_Status_Type HAL_MPWM_ProtStatClear(MPWM_Type* MPWMx, MPWM_PROTSEL prot_sel, uint32_t Data);
HAL_Status_Type HAL_MPWM_ProtIntConfig(MPWM_Type* MPWMx, MPWM_PROTSEL ProtIntCfg, FunctionalState NewState);
uint32_t HAL_MPWM_GetProtIntStat(MPWM_Type* MPWMx,MPWM_PROTSEL ProtIntCfg);

HAL_Status_Type HAL_MPWM_IntConfig(MPWM_Type* MPWMx, uint32_t MPIntCfg, FunctionalState NewState);
HAL_Status_Type HAL_MPWM_IntStatClear(MPWM_Type* MPWMx, uint32_t Type);
uint32_t HAL_MPWM_GetIntStat(MPWM_Type* MPWMx);

HAL_Status_Type HAL_MPWM_Cmd(MPWM_Type* MPWMx, uint32_t motorb, uint32_t update, uint32_t tup, uint32_t bup, uint32_t mchmode, uint32_t updown);

HAL_Status_Type HAL_MPWM_SetPeriod(MPWM_Type* MPWMx, uint32_t period);
HAL_Status_Type HAL_MPWM_SetUDuty(MPWM_Type* MPWMx, uint32_t udutyH, uint32_t udutyL);
HAL_Status_Type HAL_MPWM_SetVDuty(MPWM_Type* MPWMx, uint32_t vdutyH, uint32_t vdutyL);
HAL_Status_Type HAL_MPWM_SetWDuty(MPWM_Type* MPWMx, uint32_t wdutyH, uint32_t wdutyL);

uint32_t HAL_MPWM_GetCounter(MPWM_Type *MPWMx);

HAL_Status_Type HAL_MPWM_SetDeadTime(MPWM_Type* MPWMx, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t clkdata, uint32_t dirsel);

HAL_Status_Type HAL_MPWM_SetATR(MPWM_Type* MPWMx, MPWM_PH_VAL trgsrc,uint32_t ch, MPWM_ADCTRG_TIME atudt, MPWM_ADCTRG_MODE atmod, uint32_t atcnt);


/* Extend Mode MPWM Functions ------------------------------------------------ */
HAL_Status_Type HAL_MPWM_ExtStartCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_Ext_OPERATEVAL OperSel);
HAL_Status_Type HAL_MPWM_ExtEnableCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, uint32_t irq_n, FunctionalState NewStatus);
HAL_Status_Type HAL_MPWM_ExtPauseCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_Ext_PAUSE PauSel);

HAL_Status_Type HAL_MPWM_ExtSetPeriod(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t period);

uint32_t HAL_MPWM_ExtGetCounter(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val);

HAL_Status_Type HAL_MPWM_ExtSetDeadTime(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t dten, uint32_t pshrt, uint32_t clk, uint32_t rclkdata, uint32_t fclkdata, uint32_t dirsel);

HAL_Status_Type HAL_MPWM_ExtCaptureCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_SCAP_EDGE ScapEdge, FunctionalState NewStatus);
HAL_Status_Type HAL_MPWM_ExtClearCaptureCmd(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_CAP_MODE CapMode);
uint32_t HAL_MPWM_ExtGetCaptureCNT(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val);
uint32_t HAL_MPWM_ExtGetCaptureVal(MPWM_Type *MPWMx, MPWM_PH_VAL pha_val, MPWM_CAP_SEL CapSel);

HAL_Status_Type HAL_MPWM_ExtInit(MPWM_Type *MPWMx);

HAL_Status_Type HAL_MPWM_SetCMPProt(MPWM_Type* MPWMx, MPWM_PH_VAL pha_val, uint32_t comp, FunctionalState state);

#ifdef __cplusplus
}
#endif


#endif /* _A33M11x_MPWM_H_ */

/* --------------------------------- End Of File ------------------------------ */
