/**********************************************************************//**
* @file		A33M11x_hal_pwr.h
* @brief	Contains all macro definitions and function prototypes
* 			support for PWR firmware library on A33M11x
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

#ifndef _A33M11x_PWR_H_
#define _A33M11x_PWR_H_

/* Includes ------------------------------------------------------------------- */
#include "A33M11x.h"
#include "A33M11x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* LVI Definition ------------------------------------------------------------- */
#define LVI_DISABLE				(0x0uL<<7)
#define LVI_ENABLE					(0x1uL<<7)

#define LVI_INTDIS					(0x0uL<<5)
#define LVI_INTEN					(0x1uL<<5)
	
#define LVI_DEEPDIS				(0x0uL<<4)
#define LVI_DEEPON				(0x1uL<<4)
	
#define LVI_LVL_1_60V				(0x0uL<<0)
#define LVI_LVL_1_69V				(0x1uL<<0)
#define LVI_LVL_1_78V				(0x2uL<<0)
#define LVI_LVL_1_90V				(0x3uL<<0)
#define LVI_LVL_1_99V				(0x4uL<<0)
#define LVI_LVL_2_12V				(0x5uL<<0)
#define LVI_LVL_2_30V				(0x6uL<<0)
#define LVI_LVL_2_47V				(0x7uL<<0)
#define LVI_LVL_2_67V				(0x8uL<<0)
#define LVI_LVL_3_04V				(0x9uL<<0)
#define LVI_LVL_3_18V				(0xAuL<<0)
#define LVI_LVL_3_59V				(0xBuL<<0)
#define LVI_LVL_3_72V				(0xCuL<<0)
#define LVI_LVL_4_03V				(0xDuL<<0)
#define LVI_LVL_4_20V				(0xEuL<<0)
#define LVI_LVL_4_48V				(0xFuL<<0)

/* LVR Definition ------------------------------------------------------------- */
#define LVR_DISABLE				(0xAAuL<<8)
#define LVR_ENABLE				(0x00uL<<8)
	
#define LVR_DEEPDIS				(0x0uL<<4)
#define LVR_DEEPON				(0x1uL<<4)
	
#define LVR_1_60V				(0x0uL<<0)
#define LVR_1_69V				(0x1uL<<0)
#define LVR_1_78V				(0x2uL<<0)
#define LVR_1_90V				(0x3uL<<0)
#define LVR_1_99V				(0x4uL<<0)
#define LVR_2_12V				(0x5uL<<0)
#define LVR_2_30V				(0x6uL<<0)
#define LVR_2_47V				(0x7uL<<0)
#define LVR_2_67V				(0x8uL<<0)
#define LVR_3_04V				(0x9uL<<0)
#define LVR_3_18V				(0xAuL<<0)
#define LVR_3_59V				(0xBuL<<0)
#define LVR_3_72V				(0xCuL<<0)
#define LVR_4_03V				(0xDuL<<0)
#define LVR_4_20V				(0xEuL<<0)
#define LVR_4_48V				(0xFuL<<0)


/**
 * @brief LVI configuration structure definitions
 */
typedef struct {
	uint32_t	EnSel;
	uint32_t	IntSel;
	uint32_t	DeepSel;
	uint32_t	LvlSel;
} LVI_CFG_Type;


/**
 * @brief LVR configuration structure definitions
 */
typedef struct {
	uint32_t	EnSel;
	uint32_t	DeepSel;
	uint32_t	LvlSel;
} LVR_CFG_Type;

/* Public Functions ----------------------------------------------------------- */
void HAL_PWR_EnterSleepMode(void);
void HAL_PWR_EnterPowerDownMode(void);
void HAL_LVI_Init(LVI_CFG_Type *LVI_ConfigStruct);
void HAL_LVR_Init(LVR_CFG_Type *LVR_ConfigStruct);
#ifdef __cplusplus
}
#endif


#endif /* end _A33M11x_PWR_H_ */

/* --------------------------------- End Of File ------------------------------ */
