/**********************************************************************//**
* @file		A33M11x_hal_crc.c
* @brief	Contains all functions support for fmc firmware library
* 			on A33M11x
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
************************************************************************
* ABOV Disclaimer
*
*IMPORTANT NOTICE ? PLEASE READ CAREFULLY
*ABOV Semiconductor ("ABOV") reserves the right to make changes, corrections, enhancements, 
*modifications, and improvements to ABOV products and/or to this document at any time without notice. 
*ABOV does not give warranties as to the accuracy or completeness of the information included herein.
*Purchasers should obtain the latest relevant information of ABOV products before placing orders. 
*Purchasers are entirely responsible for the choice, selection, and use of ABOV products and 
*ABOV assumes no liability for application assistance or the design of purchaser products. No license, 
*express or implied, to any intellectual property rights is granted by ABOV herein. 
*ABOV disclaims all express and implied warranties and shall not be responsible or
*liable for any injuries or damages related to use of ABOV products in such unauthorized applications. 
*ABOV and the ABOV logo are trademarks of ABOV.
*All other product or service names are the property of their respective owners. 
*Information in this document supersedes and replaces the information previously
*supplied in any former versions of this document.
*2020 ABOV Semiconductor  All rights reserved
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A33M11x_hal_crc.h"
#include "A33M11x_hal_scu.h"
#include "A33M11x_hal_libcfg.h"



/* Public Functions ----------------------------------------------------------- */
/*********************************************************************//**
* @brief		Initializes the CRC peripheral 
* @param[in]	CRC_ConfigStruct
* @return 		HAL Status
*********************************************************************/
HAL_Status_Type HAL_CRC_Init(CRC_CFG_Type *CRC_ConfigStruct){
	uint32_t ctrl =0;

	 /* Check CRC  handle */
         if(CRC_ConfigStruct == NULL)
        {
            return HAL_ERROR;
         }
		 
	SYST_ACCESS_EN();
	
    SCU->PER2 &=~(1UL<<29);
	SCU->PCER2&=~(1UL<<29);
    /* Set up peripheral clock for CRC module */
    SCU->PER2 |= (1UL<<29);
	SCU->PCER2|= (1UL<<29);
	
	SYST_ACCESS_DIS();
	
	if(CRC_ConfigStruct->Out_inv == ENABLE){
		ctrl |= CRC_OUT_INV_EN;
	}else{
		ctrl |= CRC_OUT_INV_DIS;
	}

	if(CRC_ConfigStruct->Out_rev == ENABLE){
		ctrl |= CRC_OUT_REV_EN;
	}else{
		ctrl |= CRC_OUT_REV_DIS;
	}

	if(CRC_ConfigStruct->In_rev == ENABLE){
		ctrl |= CRC_IN_REV_DIS;
	}else{
		ctrl |= CRC_IN_REV_EN;
	}
	
	if(CRC_ConfigStruct->DMAD_Int == ENABLE){
		ctrl |= CRC_DMAD_INT_EN;
	}else{
		ctrl |= CRC_DMAD_INT_DIS;
	}
	
	ctrl |= CRC_ConfigStruct->Poly << 1;
	
	CRC->CTRL = ctrl;
   return HAL_OK;

}

/**********************************************************************//**
* @brief		Set CRC Init value
* @param[in]	init	CRC Initial value
* 					- DISABLE : Bit Disable
* 					- ENABLE: Bit Enable
* @return		HAL Status
*
**********************************************************************/
HAL_Status_Type HAL_CRC_SetInitVal(uint32_t init){
		 
	CRC->INIT = init;
	return HAL_OK;
}

/**********************************************************************//**
* @brief		Apply CRC Init value
* @param[in]	en	CRC Apply initial value
* 					- DISABLE : Bit Disable
* 					- ENABLE: Bit Enable
* @return		HAL Status
*
 **********************************************************************/
HAL_Status_Type HAL_CRC_ApplyInitVal(FunctionalState en){
	if(en == ENABLE){
		CRC->CTRL |= CRC_INIT_EN;
	}else{
		CRC->CTRL |= CRC_INIT_DIS;
	}
	return HAL_OK;
}

/**********************************************************************//**
* @brief		Get CRC Init
* @param[in]	None
* @return		CRC Initial value
*
**********************************************************************/
uint32_t HAL_CRC_GetInitVal(void){
	return CRC->INIT;
}


/**********************************************************************//**
* @brief		Set CRC Input Data8
* @param[in]	data8	CRC Input data
* @return		HAL Status
*
**********************************************************************/
HAL_Status_Type HAL_CRC_SetInputData8(uint8_t data8)
{
	CRC->IDR = data8;
	return HAL_OK;
}

/**********************************************************************//**
* @brief		Set CRC Input Data32
* @param[in]	data32	CRC Input data
* @return		HAL Status
*
**********************************************************************/
HAL_Status_Type HAL_CRC_SetInputData32(uint32_t data32)
{
	CRC->IDR = ((data32>>0)&0xff);
	CRC->IDR = ((data32>>8)&0xff);
	CRC->IDR = ((data32>>16)&0xff);
	CRC->IDR = ((data32>>24)&0xff);
	return HAL_OK;
}


/**********************************************************************//**
* @brief		Get CRC Output Data
* @param[in]	None
* @return		CRC Output Data
*
**********************************************************************/
uint32_t HAL_CRC_GetOutputData(void)
{
	return CRC->ODR;

}

/**********************************************************************//**
* @brief		Checks whether the specified CRC status flag is set or not
* @param[in]	None
* @return		CRC status flag
 **********************************************************************/
uint32_t HAL_CRC_GetStatus(void)
{
	return CRC->STAT;

}

/**********************************************************************//**
* @brief		Clear specified pending in CRC peripheral
* @param[in]	Type	Interrupt pending to clear
*						- CRC_DMAD_INT
* @return		HAL Status
**********************************************************************/
HAL_Status_Type HAL_CRC_ClearPending(uint32_t Type){
	CRC->STAT = Type;
	return HAL_OK;
}

