/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC0_IRQHandler_IT(void);
void TIMER0_IRQHandler_IT(void);
void TIMER1_IRQHandler_IT(void);
void TIMER2_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void TIMER_Configure(void);
void GPIO_Configure(void);
void ADC_Configure(void);
void ADC_MultipleRun(void);
void ADC_MessageRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static uint32_t fflag;

static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t Using ADC in Multiple mode \n\r"
"************************************************\n\r";

static TIMER_PERIODICCFG_Type TIMERa_Config;
static TIMER_PERIODICCFG_Type TIMERb_Config;
static TIMER_PERIODICCFG_Type TIMERc_Config;
static ADC_CFG_Type AD_config;

/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC0_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC0);

	if ((status & ADC_STAT_SEQ)==ADC_STAT_SEQ){	// Sequence Conversion End Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_SEQ);	
	}
	if ((status & ADC_STAT_TRIGGER)==ADC_STAT_TRIGGER){	// ADC Trigger Interrupt Flag Check
		fflag=1;
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_TRIGGER);	
		PB->ODR ^= 0x01;
	}
}

/*******************************************************************************
* @brief      TIMER0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER0) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(TIMER0, TIMER_SR_MFB);
	}
}

/*******************************************************************************
* @brief      TIMER1 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER1_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER1) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(TIMER1, TIMER_SR_MFB);
	}
}

/*******************************************************************************
* @brief      TIMER2 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER2_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER2) & TIMER_SR_MFB) == TIMER_SR_MFB){
		HAL_TIMER_ClearStatus(TIMER2, TIMER_SR_MFB);
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG
		_DBG("CH0 = "); _DBH16(ADC0->DR0>>4); _DBG(" ");
		_DBG("CH2 = "); _DBH16(ADC0->DR1>>4); _DBG(" ");
		_DBG("CH4 = "); _DBH16(ADC0->DR2>>4); _DBG(" ");			
		_DBG("\r\n");
	 #endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
		// Toggle Port Setting
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 1, PB1_MUX_PB1);
	HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 1, PULL_UP_DOWN_DISABLE);	
	
	
	//Analog Input Port (AN0/AN2/AN4) Setting
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_AN0);
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_AN2);
	HAL_GPIO_ConfigOutput(PA, 2, INPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 4, PA4_MUX_AN4);
	HAL_GPIO_ConfigOutput(PA, 4, INPUT);
	HAL_GPIO_ConfigPullup(PA, 4, PULL_UP_DOWN_DISABLE);	

}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{	

	// ADC Configuration - Multiple Mode, 3 Sequence, Timer Trigger Mode, ADC Clock : 32/8=4MHz
	AD_config.Mode = ADC_MULTI_MODE;
	AD_config.SamplingTime=0x1f; 	// Conversion Speed = (Sampling Time + 12 Cycle + 4 Cycle)
	AD_config.SeqCnt = 3;
	AD_config.RestartEn = 0;
	AD_config.TrgSel = ADC_TRIGGER_TIMER;
	
	AD_config.UseClk = ADC_EXTERNAL_CLK;
	
	HAL_SCU_MiscClockConfig(4, ADC_TYPE, SCU_MCCR_CSEL_HSI, 8);
	
	if( HAL_ADC_Init(ADC0, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}

	// trigger =  timer0
	HAL_ADC_TriggerlSel(ADC0, 0xFFFFF120); //Trigger Selection, 0 : Timer0, 1: Timer2, 2: Timer1
	
	HAL_ADC_ChannelSel_1(ADC0, 0x00040200);	// ch0-AN0, ch1-AN2, ch2-AN4
	
	HAL_ADC_ConfigInterrupt(ADC0, (ADC_INTEN_SEQ | ADC_INTEN_TRIGGER), ENABLE);	// ADC Seqeunce, Trigger Interrupt Enable

	// Enable Interrupt for ADC0 channel 
	NVIC_SetPriority(ADC0_IRQn, 3);
	NVIC_EnableIRQ(ADC0_IRQn);
	
}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{	
	//------------------------------------------------------
	// TIMER 0 Setting
	//------------------------------------------------------

	TIMERa_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	TIMERa_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	TIMERa_Config.GRA = (500); 	// adc trigger signal 
	TIMERa_Config.GRB = (10000); // 10msec			
	TIMERa_Config.Cnt=0;		
	TIMERa_Config.StartLevel=START_LOW;
	TIMERa_Config.AdcTrgEn=ENABLE;
	
	// Initialize Timern (TIMER0)
	HAL_TIMER_Init(TIMER0, PERIODIC_MODE, &TIMERa_Config);  
	// Timer0 Match B Interrupt Enable
	HAL_TIMER_ConfigInterrupt(TIMER0, TIMER_INTCFG_MBIE, ENABLE);

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
	NVIC_EnableIRQ(TIMER0_IRQn);	
	
	//------------------------------------------------------
	// TIMER 1 Setting
	//------------------------------------------------------

	TIMERb_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	TIMERb_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	TIMERb_Config.GRA = (500); 	// adc trigger signal 
	TIMERb_Config.GRB = (10000); // 10msec			
	TIMERb_Config.Cnt=0;		
	TIMERb_Config.StartLevel=START_LOW;
	TIMERb_Config.AdcTrgEn=ENABLE;
	
	// Initialize Timern (TIMER1)
	HAL_TIMER_Init(TIMER1, PERIODIC_MODE, &TIMERb_Config); 
	
	// Timer1 Match B Interrupt Enable 
	HAL_TIMER_ConfigInterrupt(TIMER1, TIMER_INTCFG_MBIE, ENABLE);

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER1_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
	NVIC_EnableIRQ(TIMER1_IRQn);
	
	//------------------------------------------------------
	// TIMER 2 Setting
	//------------------------------------------------------

	TIMERc_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	TIMERc_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	TIMERc_Config.GRA = (500); 	// adc trigger signal 
	TIMERc_Config.GRB = (10000); // 10msec			
	TIMERc_Config.Cnt=0;		
	TIMERc_Config.StartLevel=START_LOW;
	TIMERc_Config.AdcTrgEn=ENABLE;

	// Initialize Timern (TIMER2)	
	HAL_TIMER_Init(TIMER2, PERIODIC_MODE, &TIMERc_Config); 
	
	HAL_TIMER_ConfigInterrupt(TIMER2, TIMER_INTCFG_MBIE, ENABLE);

  	/* preemption = 1, sub-priority = 1 */
  	NVIC_SetPriority(TIMER2_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
	  NVIC_EnableIRQ(TIMER2_IRQn);
	
	// TIMER2 Start
	HAL_TIMER_Cmd(TIMER2, ENABLE);
	
	__NOP();
	__NOP();
	__NOP();
	
	// TIMER0 Start
	HAL_TIMER_Cmd(TIMER0, ENABLE);
	
	__NOP();
	__NOP();
	__NOP();
	
	// TIMER1 Start
	HAL_TIMER_Cmd(TIMER1, ENABLE);
	
	__NOP();
	__NOP();
	__NOP();
}

/**********************************************************************
 * @brief		ADC_MultipleRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_MultipleRun(void)
{	
		// ADC trigger interrupt check
		fflag=0;
		while(fflag==0){}
			
		UART_DebugMessage();
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*ADC Configure*/
	ADC_Configure(); 
  
	/*TIMER Configure*/
	TIMER_Configure(); 
	
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*ADC start*/
		ADC_MultipleRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
