/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void COMP_Configure( void);
void AFE_CompInterruptRun(void);
void AFE0_IRQHandler_IT(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" COMP Interrupt test \n\r"
"************************************************\n\r";
static uint8_t flag;
/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG

	#endif

}


/*******************************************************************************
* @brief      AFE0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void AFE0_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_AFE_COMP_GetStatus();		// AFE0 Interrupt status check 
	
	if ((status & AFE_CMPSR_CMP0INTF) == AFE_CMPSR_CMP0INTF)	// AFE0 Interrupt occur 
	{		
		HAL_AFE_COMP_ClearStatus(AFE_CMPSR_CMP0INTF);
		flag=1;
	}
}
/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// PB0  test pin setting
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);
	
	// PA0,1,2 Comparator0 pin setting
	// CO0(PA0)
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_CO0);
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	// AIN0(PA1)
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_AIN0);
	HAL_GPIO_ConfigOutput(PA, 1, INPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);	
	// AIP0(PA2)	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_AIP0);
	HAL_GPIO_ConfigOutput(PA, 2, INPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);
}

/**********************************************************************
 * @brief		COMP_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void COMP_Configure(void)
{
	HAL_AFE_ClockInit(ENABLE); // Comparator Peri Clock Enable
	
	HAL_AFE_COMP_DebounceFilter(0, 10); // 1/8MHz = 125ns, 125ns * 10 = 1.25us
	
	HAL_AFE_COMP_IntConfigure(0, INTPOL_HIGH, INTTYPE_LEVEL); // Comp0, high level interrupt setting
	
	HAL_AFE_COMP_IntCmd(0, ENABLE);		// Comp0, Interrupt Enable

	HAL_AFE_COMP_Cmd(0, ENABLE); //Comparator0 enable
	
	HAL_AFE_COMP_ClearStatusAll();
	
	NVIC_SetPriority(AFE0_IRQn, 0);
	NVIC_EnableIRQ(AFE0_IRQn);
	
}

/**********************************************************************
 * @brief		AFE_CompPollingRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void AFE_CompInterruptRun(void)
{	
	flag=0;
	if(flag==1) {
		_DBG("AFE0 Interrupt Occur !! \r\n");
		flag=0;
	}
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*COMP Configure*/
	COMP_Configure();
  
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/* AFE CompInterruptRun  */
	   AFE_CompInterruptRun(); 
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
