/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC0_IRQHandler_IT(void);
void ADC1_IRQHandler_IT(void);
void TIMER0_IRQHandler_IT(void);
void TIMER1_IRQHandler_IT(void);
void TIMER2_IRQHandler_IT(void);
void TIMER3_IRQHandler_IT(void);

void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void ADC_Configure(void);
void TIMER_Configure(void);
void DMA_Configure(void);
void ADC_DMATestRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static ADC_CFG_Type AD_config;
static TIMER_PERIODICCFG_Type TIMER0_Config;
static 	DMA_CFG_Type	DMA_Config;
	uint32_t dma_rx, dma_add;
	uint32_t data0[200],data1[200] ={0,};	

static uint32_t fflag0;
static uint32_t fflag1;

static const uint8_t menu[] =
"\r\n\r\n************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t DMA ADC FUNCTION TEST ...  \n\r"
"\t Start \n\r"
"************************************************\n\r";


/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      ADC0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC0_IRQHandler_IT(void)
{
	uint16_t status;
	
	PORT_ACCESS_EN();
	
	status = HAL_ADC_GetStatus(ADC0);

	if ((status & ADC_STAT_SEQ)==ADC_STAT_SEQ){	// Sequence Conversion End Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_SEQ);
		fflag0=1;
	}
	
		if ((status & ADC_STAT_TRIGGER)==ADC_STAT_TRIGGER){	// Trigger Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_TRIGGER);
	}	
		
	HAL_ADC_ClearStatus(ADC0, ADC_STAT_SINGLE);
}

/*******************************************************************************
* @brief      ADC1 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC1_IRQHandler_IT(void)
{
	uint16_t status;
	
	PORT_ACCESS_EN();
	
	status = HAL_ADC_GetStatus(ADC1);

	if ((status & ADC_STAT_SEQ)==ADC_STAT_SEQ){	// Sequence Conversion End Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC1, ADC_STAT_SEQ);
		fflag1=1;
	}
	
	if ((status & ADC_STAT_TRIGGER)==ADC_STAT_TRIGGER){	// Trigger Interrupt Flag Check
		HAL_ADC_ClearStatus(ADC1, ADC_STAT_TRIGGER);
	}
			
	HAL_ADC_ClearStatus(ADC0, ADC_STAT_SINGLE);
}

/*******************************************************************************
* @brief      TIMER0 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER0_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER0) & TIMER_SR_MFB) == TIMER_SR_MFB){	// Timer0 Match B Interrupt Flag Check
		HAL_TIMER_ClearStatus(TIMER0, TIMER_SR_MFB);
	}
}

/*******************************************************************************
* @brief      TIMER1 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER1_IRQHandler_IT(void)
{
	if ((HAL_TIMER_GetStatus(TIMER1) & TIMER_SR_MFB) == TIMER_SR_MFB){	// Timer1 Match B Interrupt Flag Check
		HAL_TIMER_ClearStatus(TIMER1, TIMER_SR_MFB);
	}
}

/*******************************************************************************
* @brief      TIMER2 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER2_IRQHandler_IT(void)
{
		if ((HAL_TIMER_GetStatus(TIMER2) & TIMER_SR_MFB) == TIMER_SR_MFB){	// Timer2 Match B Interrupt Flag Check
		HAL_TIMER_ClearStatus(TIMER2, TIMER_SR_MFB);
	}
}

/*******************************************************************************
* @brief      TIMER3 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER3_IRQHandler_IT(void)
{
		if ((HAL_TIMER_GetStatus(TIMER3) & TIMER_SR_MFB) == TIMER_SR_MFB){	// Timer3 Match B Interrupt Flag Check
		HAL_TIMER_ClearStatus(TIMER3, TIMER_SR_MFB);
	}
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	//Test Port Setting
	HAL_GPIO_ConfigureFunction(PC, 2, PC2_MUX_PC2);
	HAL_GPIO_ConfigOutput(PC, 2, PUSH_PULL_OUTPUT);
	
	//Analog Input Port Setting (AN0-AN3)
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_AN0);
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_AN1);
	HAL_GPIO_ConfigOutput(PA, 1, INPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_AN2);
	HAL_GPIO_ConfigOutput(PA, 2, INPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);	
	
	HAL_GPIO_ConfigureFunction(PA, 3, PA3_MUX_AN3);
	HAL_GPIO_ConfigOutput(PA, 3, INPUT);
	HAL_GPIO_ConfigPullup(PA, 3, PULL_UP_DOWN_DISABLE);	
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{	

	// ADC Configuration - Sequence Mode, ADC Clock : 32/8=4MHz, Timer Trigger Enable
	AD_config.Mode = ADC_SINGLE_MODE;
	AD_config.SamplingTime = 1;
	AD_config.SeqCnt = 4;
	AD_config.RestartEn = 1;
	AD_config.DmaOpt = 1;
	AD_config.TrgSel = ADC_TRIGGER_TIMER;
	
	AD_config.UseClk = ADC_EXTERNAL_CLK;

	HAL_SCU_MiscClockConfig(4, ADC_TYPE, SCU_MCCR_CSEL_HSI, 8);
	
	if( HAL_ADC_Init(ADC0, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}
	if( HAL_ADC_Init(ADC1, &AD_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}

	// trigger timer 0,1,2,3
	HAL_ADC_TriggerlSel(ADC0, 0xFFFF3210); 
	HAL_ADC_TriggerlSel(ADC1, 0xFFFF3210); 
		
	// ch0, ch1, ch2, ch3
	HAL_ADC_ChannelSel(ADC0, 0x03020100);	
	HAL_ADC_ChannelSel(ADC1, 0x03020100);	
		
	// adc Sequence interrupt enable
	HAL_ADC_ConfigInterrupt(ADC0, ADC_INTEN_SEQ, ENABLE);	
	HAL_ADC_ConfigInterrupt(ADC1, ADC_INTEN_SEQ, ENABLE);	
	
	// Enable Interrupt for ADC0,1 channel 
	NVIC_SetPriority(ADC0_IRQn, 3);
	NVIC_EnableIRQ(ADC0_IRQn);
	NVIC_SetPriority(ADC1_IRQn, 3);
	NVIC_EnableIRQ(ADC1_IRQn);
	
}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	// Timer0,1,2,3 Setting
	TIMER0_Config.CkSel = PCLK_2; // SystemPeriClock=8Mhz=PCLK  CLOCK_IN = PCLK/2=4Mhz 
	TIMER0_Config.Prescaler = 3;    // TCLK = (CLOCK_IN / (PRS+1))  = 4Mhz / 4 = 1Mhz ->1us

	TIMER0_Config.GRB = (2000); // 2msec			
	TIMER0_Config.Cnt=0;		
	TIMER0_Config.StartLevel=START_LOW;
	TIMER0_Config.AdcTrgEn=ENABLE;
	
	TIMER0_Config.GRA = (300); 	// adc trigger signal 
	if((HAL_TIMER_Init(TIMER0, PERIODIC_MODE, &TIMER0_Config))!=HAL_OK)	// timer0 setting - GRA:300,GRB:2000 
	{
			/* Initialization Error */
     Error_Handler();
	} 
	
	TIMER0_Config.GRA = (500);
	if((HAL_TIMER_Init(TIMER1, PERIODIC_MODE, &TIMER0_Config))!=HAL_OK)	// timer1 setting - GRA:500,GRB:2000 
	{
			/* Initialization Error */
     Error_Handler();
	} 

	TIMER0_Config.GRA = (900);
		if((HAL_TIMER_Init(TIMER2, PERIODIC_MODE, &TIMER0_Config))!=HAL_OK)	// timer2 setting - GRA:900,GRB:2000 
	{
			/* Initialization Error */
     Error_Handler();
	} 
	
	TIMER0_Config.GRA = (1100);
	if((HAL_TIMER_Init(TIMER3, PERIODIC_MODE, &TIMER0_Config))!=HAL_OK)	// timer3 setting - GRA:1100,GRB:2000 
	{
			/* Initialization Error */
     Error_Handler();
	} 

	/* Timer 0,1,2,3 GRB match Interrupt */
	HAL_TIMER_ConfigInterrupt(TIMER0, TIMER_INTCFG_MBIE, ENABLE);
	HAL_TIMER_ConfigInterrupt(TIMER1, TIMER_INTCFG_MBIE, ENABLE);
	HAL_TIMER_ConfigInterrupt(TIMER2, TIMER_INTCFG_MBIE, ENABLE);
	HAL_TIMER_ConfigInterrupt(TIMER3, TIMER_INTCFG_MBIE, ENABLE);
	
	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER0_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER0 channel */
	NVIC_EnableIRQ(TIMER0_IRQn);	
	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER1_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER1 channel */
	NVIC_EnableIRQ(TIMER1_IRQn);	
	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(TIMER2_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER2 channel */
	NVIC_EnableIRQ(TIMER2_IRQn);
	NVIC_SetPriority(TIMER3_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for TIMER2 channel */
	NVIC_EnableIRQ(TIMER3_IRQn);

	// timer 0,1,2,3 start
	HAL_TIMER_Cmd(TIMER0, ENABLE); 
	HAL_TIMER_Cmd(TIMER1, ENABLE); 
	HAL_TIMER_Cmd(TIMER2, ENABLE); 	
	HAL_TIMER_Cmd(TIMER3, ENABLE); 
}


/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{
	/* DMA0,1 TEST */
	// dma peri clock enable
	if(HAL_DMAC_Init()!=HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();		
	}
	
	// dma configuratio : adc0_rx-0x4000B02C, count-200, size-word, dir-peri->memory
	dma_rx = ADC0_RX;
	dma_add = 0x4000B02C;
	
	DMA_Config.transcnt = 20;
	DMA_Config.perisel = dma_rx;
	DMA_Config.bussize = DMA_CR_WORD_TRANS;
	DMA_Config.dirsel = DMA_CR_DIR_PERI_TO_MEM;
	DMA_Config.periaddr =dma_add;
	DMA_Config.memaddr = (uint32_t)data0;
	
	// DMA0 configure setting
	HAL_DMAC_Setup(DMA0, &DMA_Config);
	
	dma_rx = ADC1_RX;
	dma_add = 0x4000B12C;
	
	DMA_Config.transcnt = 20;
	DMA_Config.perisel = dma_rx;
	DMA_Config.bussize = DMA_CR_WORD_TRANS;
	DMA_Config.dirsel = DMA_CR_DIR_PERI_TO_MEM;
	DMA_Config.periaddr =dma_add;
	DMA_Config.memaddr = (uint32_t)data1;
	
	// DMA1 configure setting
	HAL_DMAC_Setup(DMA1, &DMA_Config);

	// DMA0, 1 channel start
	HAL_DMAC_ChCmd(DMA0, ENABLE);
	HAL_DMAC_ChCmd(DMA1, ENABLE);		

	/* DMA TEST END */
}
/**********************************************************************
 * @brief		ADC_InterruptRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_DmaTestRun(void)
{	
	int i,j;
	
	if (AD_config.RestartEn==0) {
			HAL_ADC_Start(ADC0); // start 	
			HAL_ADC_Start(ADC1); // start 
		}
		for(i=0;i<20;i=i+4)
		{
			fflag0=0;
			while(fflag0==0){}	 // ADC0 Sequence Interrupt Flag Check
	#ifdef _DEBUG_MSG
		_DBG("\t test ADC0 Module !! \t \r\n");
		_DBG("AN0 = "); _DBH16(ADC0->DR0>>4); _DBG(" / ");_DBG("DR0 =0x"); _DBH32(ADC0->DR0); _DBG("\t"); _DBH32(data0[i]); _DBG("\r\n");
		_DBG("AN1 = "); _DBH16(ADC0->DR1>>4); _DBG(" / ");_DBG("DR1 =0x"); _DBH32(ADC0->DR1); _DBG("\t"); _DBH32(data0[i+1]); _DBG("\r\n");
		_DBG("AN2 = "); _DBH16(ADC0->DR2>>4); _DBG(" / ");_DBG("DR2 =0x"); _DBH32(ADC0->DR2); _DBG("\t"); _DBH32(data0[i+2]); _DBG("\r\n");
		_DBG("AN3 = "); _DBH16(ADC0->DR3>>4); _DBG(" / ");_DBG("DR3 =0x"); _DBH32(ADC0->DR3); _DBG("\t"); _DBH32(data0[i+3]); _DBG("\r\n");			
		_DBG("\r\n");
				
			for(j=0;j<0x10000;j++);
	#endif

		fflag1=0;
		while(fflag1==0){}	// ADC1 Sequence Interrupt Flag Check
	#ifdef _DEBUG_MSG
		_DBG("\t test ADC1 Module !! \t \r\n");
		_DBG("AN0 = "); _DBH16(ADC1->DR0>>4); _DBG(" / ");_DBG("DR0 =0x"); _DBH32(ADC1->DR0); _DBG("\t"); _DBH32(data1[i]); _DBG("\r\n");
		_DBG("AN1 = "); _DBH16(ADC1->DR1>>4); _DBG(" / ");_DBG("DR1 =0x"); _DBH32(ADC1->DR1); _DBG("\t"); _DBH32(data1[i+1]); _DBG("\r\n");
		_DBG("AN2 = "); _DBH16(ADC1->DR2>>4); _DBG(" / ");_DBG("DR2 =0x"); _DBH32(ADC1->DR2); _DBG("\t"); _DBH32(data1[i+2]); _DBG("\r\n");
		_DBG("AN3 = "); _DBH16(ADC1->DR3>>4); _DBG(" / ");_DBG("DR3 =0x"); _DBH32(ADC1->DR3); _DBG("\t"); _DBH32(data1[i+3]); _DBG("\r\n");			
		_DBG("\r\n");
		
		for(j=0;j<0x10000;j++);
		}
	#endif
		
			HAL_ADC_DeInit(ADC0); // close 	
			HAL_ADC_DeInit(ADC1); // close
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*DMA Configure*/
	DMA_Configure();
	
	/*ADC Configure*/
	ADC_Configure(); 
	
	/*TIMER Configure*/
	TIMER_Configure(); 
  
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*ADC DMA Test start*/
		ADC_DmaTestRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
