/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SPI_Configure(void);
void SPI_DMATestRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static SPI_CFG_Type SPS_config;
static SPI_CFG_Type SPM_config;
static DMA_CFG_Type	DMA_Config;	
static uint32_t rx_data[10]={0,};
static uint32_t i;
static uint8_t ch;

static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t SPI DMA TEST \n\r"
"************************************************\n\r";
	
/* Public Function ------------------------------------------------------------*/


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// MASTER : SPI0(SS0, SCK0, MOSI0, MISO0) PIN FUNCTION SETTING
	HAL_GPIO_ConfigureFunction(PA, 12, PA12_MUX_SS0);
	HAL_GPIO_ConfigOutput(PA, 12, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 12, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 13, PA13_MUX_SCK0);
	HAL_GPIO_ConfigOutput(PA, 13, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 13, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 14, PA14_MUX_MOSI0);
	HAL_GPIO_ConfigOutput(PA, 14, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 14, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 15, PA15_MUX_MISO0);
	HAL_GPIO_ConfigOutput(PA, 15, INPUT);
	HAL_GPIO_ConfigPullup(PA, 15, PULL_UP_ENABLE);

	// SLAVE : SPI1(SS1, SCK1, MOSI1, MISO1) PIN FUNCTION SETTING
	HAL_GPIO_ConfigureFunction(PB, 10, PB10_MUX_SS1);
	HAL_GPIO_ConfigOutput(PB, 10, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 10, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 11, PB11_MUX_SCK1);
	HAL_GPIO_ConfigOutput(PB, 11, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 11, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 12, PB12_MUX_MOSI1);
	HAL_GPIO_ConfigOutput(PB, 12, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 12, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 13, PB13_MUX_MISO1);
	HAL_GPIO_ConfigOutput(PB, 13, INPUT);
	HAL_GPIO_ConfigPullup(PB, 13, PULL_UP_ENABLE); 
}

/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_Configure(void)
{	
	// SPI0 Configuration : Data 8bit, CPHA:0, CPOL:0, MSB, Master mode, clock : 100kHz
	SPM_config.Databit = SPI_DS_8BITS;
	SPM_config.CPHA = SPI_CPHA_LO;
	SPM_config.CPOL = SPI_CPOL_LO;
	SPM_config.DataDir = SPI_MSB_FIRST;
	SPM_config.Mode = SPI_MASTER_MODE;
	SPM_config.BaudRate = 79; // PCLK / (79+1) 

	if(HAL_SPI_Init(SPI0, &SPM_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();		
	}
	// SPI0 enable
	HAL_SPI_Enable(SPI0, ENABLE);   
	
	// SPI1 Configuration : Data 8bit, CPHA:0, CPOL:0, MSB, Slave mode, clock : 100kHz
	SPS_config.Databit = SPI_DS_8BITS;
	SPS_config.CPHA = SPI_CPHA_LO;
	SPS_config.CPOL = SPI_CPOL_LO;
	SPS_config.DataDir = SPI_MSB_FIRST;
	SPS_config.Mode = SPI_SLAVE_MODE;
	SPS_config.BaudRate = 79; // PCLK / (2+1) 
		
	if(HAL_SPI_Init(SPI1, &SPS_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();		
	}
	
	// SPI1 enable
	HAL_SPI_Enable(SPI1, ENABLE);  
	
}

/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{	
	// dma peri clock enable
	HAL_DMAC_Init();
	
	/* DMA RX TEST */
	DMA_Config.transcnt = 10;
	DMA_Config.perisel = SPI1_RX;
	DMA_Config.bussize = DMA_CR_WORD_TRANS;
	DMA_Config.dirsel = DMA_CR_DIR_PERI_TO_MEM;
	DMA_Config.periaddr =(0x40009000+(1*0x100));
	DMA_Config.memaddr = (uint32_t)rx_data;
	
	// dma0 configure setting
	HAL_DMAC_Setup(DMA0, &DMA_Config);
	
	// dma0 start
	HAL_DMAC_ChCmd(DMA0,ENABLE);
}
/**********************************************************************
 * @brief		SPI_DmaTestRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_DmaTestRun(void)
{	

	// spi buffer clear
	HAL_SPI_ClearBuffer(SPI0, (SPI_CR_TXBC|SPI_CR_RXBC));
	HAL_SPI_ClearBuffer(SPI1, (SPI_CR_TXBC|SPI_CR_RXBC));

	// dma0 configure setting
	HAL_DMAC_Setup(DMA0, &DMA_Config);
	
	// dma0 start
	HAL_DMAC_ChCmd(DMA0,ENABLE);
	
	for(i=0;i<15;i++)
	{
		// check tx buffer empty 
		while((HAL_SPI_GetStatus(SPI0) & SPI_STAT_TXBUF_EMPTY) != SPI_STAT_TXBUF_EMPTY); 
			
		if(ch==0) HAL_SPI_TransmitData(SPI0, 0x61+i);
		else if(ch==1) HAL_SPI_TransmitData(SPI0, 0x30+i);
		else if(ch==2) HAL_SPI_TransmitData(SPI0, 0x41+i);
		else if(ch==3) HAL_SPI_TransmitData(SPI0, 0x21+i);	
		}

		// check dma end of transfers 
		while((HAL_DMAC_GetStatus(DMA0))!=SET);
		
#ifdef _DEBUG_MSG
		for(i=0;i<10;i++)
		{
			_DBH(rx_data[i]);
		}
			
		_DBG("\t DMA0 SPI RX  SUCCESS !! \r\n");
#endif
		
		ch++;
			
		if(ch>3) ch=0;	

}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*SPI Configure*/
	SPI_Configure(); 
  
	/*DMA Configure*/
	DMA_Configure(); 
	
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*SPI DMA Test start*/
		SPI_DmaTestRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
