/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void UART_Configure(void);
void DMA_Configure(void);
void UART_DMATestRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static uint32_t uart_rx[10] = {0,};
static UART_CFG_Type UARTConfigStruct;
static DMA_CFG_Type	DMA_Config;
static uint8_t ch, i;

static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t UART DMA TEST \n\r"
"************************************************\n\r";
	
/* Public Function ------------------------------------------------------------*/


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*
	 * Initialize UART0 pin connect
	 */
	HAL_GPIO_ConfigureFunction(PC, 14, PC14_MUX_RXD0);
	HAL_GPIO_ConfigOutput(PC, 14, INPUT);
	HAL_GPIO_ConfigPullup(PC, 14, PULL_UP_ENABLE);
	
	HAL_GPIO_ConfigureFunction(PC, 15, PC15_MUX_TXD0);
	HAL_GPIO_ConfigOutput(PC, 15, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PC, 15, PULL_UP_ENABLE);	
	
	/*
	 * Initialize UART1 pin connect
	 */
	HAL_GPIO_ConfigureFunction(PC, 5, PC5_MUX_RXD1);
	HAL_GPIO_ConfigOutput(PC, 5, INPUT);
	HAL_GPIO_ConfigPullup(PC, 5, PULL_UP_ENABLE);
	
	HAL_GPIO_ConfigureFunction(PC, 6, PC6_MUX_TXD1);
	HAL_GPIO_ConfigOutput(PC, 6, PUSH_PULL_OUTPUT);	
	HAL_GPIO_ConfigPullup(PC, 6, PULL_UP_ENABLE);	
}

/**********************************************************************
 * @brief		UART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{	
	/* Initialize UART Configuration parameter structure to default state:
	 * Baudrate = 38400bps
	 * 8 data bit
	 * 1 Stop bit
	 * None parity
	 */
	HAL_UART_ConfigStructInit(&UARTConfigStruct);
	
	// Initialize DEBUG_UART_PORT peripheral with given to corresponding parameter
	HAL_UART_Init(UART0, &UARTConfigStruct);
	HAL_UART_Init(UART1, &UARTConfigStruct);
}

/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{	
	// dma peri clock enable
	HAL_DMAC_Init();
	
	/* DMA RX TEST */
	DMA_Config.transcnt = 10;
	DMA_Config.perisel = UART1_RX;
	DMA_Config.bussize = DMA_CR_WORD_TRANS;
	DMA_Config.dirsel = DMA_CR_DIR_PERI_TO_MEM;
	DMA_Config.periaddr =0x40008100;	
	DMA_Config.memaddr = (uint32_t)uart_rx;
	
	// dma0 configure setting
	HAL_DMAC_Setup(DMA0, &DMA_Config);
	
	// dma0 start
	HAL_DMAC_ChCmd(DMA0,ENABLE);
}

/**********************************************************************
 * @brief		UART_DmaTestRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DmaTestRun(void)
{		
	for(i=0;i<15;i++)
	{
		while (!(UART1->LSR & UART_LSR_THRE));
				
		if(ch==0) HAL_UART_TransmitByte(UART1,0x61 + i);
		else if(ch==1) HAL_UART_TransmitByte(UART1,0x30 + i);
		else if(ch==2) HAL_UART_TransmitByte(UART1,0x41 + i);
		else if(ch==3) HAL_UART_TransmitByte(UART1,0x21 + i);	
	}
	
	while((HAL_DMAC_GetStatus(DMA0))!=SET);
	
#ifdef _DEBUG_MSG
	for(i=0;i<10;i++)
	{
		_DBH32(uart_rx[i]); _DBG("\r\n");
	}
			
	_DBG("\t DMA0 UART RX  SUCCESS !! \r\n");
#endif

	ch++;
			
	if(ch>3) ch=0;		
	
	// Dummy reading
	while (HAL_UART_GetLineStatus(UART1) & UART_LSR_RDR)
	{
		HAL_UART_ReceiveByte(UART1);
	}
	
	// dma0 configure setting
	HAL_DMAC_Setup(DMA0, &DMA_Config);
	
	// dma0 start
	HAL_DMAC_ChCmd(DMA0,ENABLE);
	
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*UART Configure*/
	UART_Configure(); 
  
	/*DMA Configure*/
	DMA_Configure(); 
	
	 /* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*UART DMA Test start*/
		UART_DmaTestRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
