/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define size				512	// 1page

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void CFMC_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void CFMC_WriteEraseRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t cmdm[] =
"A33M11x> ";
static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t Code Flash Test  \n\r"
"\t 1. Flash Init  \n\r"
"\t 2. Flash Erase[512B, 0x1F000~0x1F200]  \n\r"
"\t 3. Flash Write[512B, 0x1F000~0x1F200] \n\r"
"\t 4. Flash Verify[512B, 0x1F000~0x1F200]  \n\r"
"************************************************\n\r";
	
volatile uint8_t		chkint_cnt = 0;
static uint8_t ch_rtn, cmdn_cnt=0, WaitVal;
static uint8_t mod_check[5] = {0x30, 0x31, 0x32, 0x33, 0x34};
	
static int	i, tmp;
static unsigned char	fbuffer[size];
static unsigned long	result; 


/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      Code Flash Memory Controller interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void CFMC_IRQHandler_IT(void)
{
	HAL_CFMC_ClearAccessStatus(CFMC_STAT_WDONE);

	#ifdef _DEBUG_MSG	
	_DBG("Interrupt Done !\r\n");
	#endif

}
/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		CFMC_WriteEraseRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CFMC_WriteEraseRun(void)
{		
	if (cmdn_cnt==0)
		{
			_DBG(cmdm);
			cmdn_cnt=1;
		}
		
		ch_rtn = UARTGetChar(UART0);
		
		_DBC(ch_rtn);
		_DBG("\r\n");
		
		if (ch_rtn == mod_check[1])		// Flash Init
		{
			//-------------------------------------------------------
			// Get Test Value Line
			//-------------------------------------------------------
			// Get Wait Value
			_DBG("Wait Value(0~7) >");
			
			ch_rtn = UARTGetChar(UART0);
			
			_DBC(ch_rtn);
			_DBG("\r\n");
			
			WaitVal = ch_rtn;
			
			HAL_CFMC_WaitCmd(WaitVal-0x30);	// Code Flash Wait Setting
			
				// Get Cache Selection
			_DBG("Cache Selection (0:Disable, 1:Enable) >");
			
			ch_rtn = _DG;

			_DBC(ch_rtn);
			_DBG("\r\n");

			if (ch_rtn == 0x30)
			{
				HAL_CFMC_CacheCmd(DISABLE);	// code & instruction cache disable
			}
			else 
			{
				HAL_CFMC_CacheCmd(ENABLE);	// code & instruction cache enable
			}
			
			_DBG("Interrupt Config (0:Disable, 1:Enable) > ");
			
			ch_rtn = UARTGetChar(UART0);
			
			_DBC(ch_rtn);
			_DBG("\r\n");
			
			if (ch_rtn == 0x31)
			{
				HAL_CFMC_InterruptCmd(ENABLE);
				__enable_irq();
			}
			else 
			{
				HAL_CFMC_InterruptCmd(DISABLE);
				__disable_irq();
			}
			
			// Flash Unlock
			HAL_CFMC_FlashAccessCmd(ENABLE);
			
			// Code Flash Write Enable
			HAL_CFMC_WriteCmd(ENABLE);
			
			cmdn_cnt=0;
		}
		else if (ch_rtn == mod_check[2])		// Flash Page Erase
		{
			
			HAL_CFMC_Erase(CFMC_CTRL_PERS,0x1F000);
			
			
			for(i=0; i<20; i++)
			{
				_DBG(" .");
				
				for (tmp=0; tmp<0x2FF; tmp++);
			}
			
			_DBG("\r\n Erase Done !!! \r\n");
			
			cmdn_cnt=0;
		}
		else if (ch_rtn == mod_check[3])		// Flash Write
		{
			
			for(i=0;i<size;i++) {
				if (i%2 == 0)
				{
					fbuffer[i] = 0xAA;
				}
				else
				{
					fbuffer[i] = 0x55;
				}
			}
			
			__NOP();
			__NOP();
			__NOP();
			__NOP();
			
			__disable_irq();
			HAL_CFMC_ProgramPage(0x1F000, size, fbuffer);
			__enable_irq();
			
			for(i=0; i<20; i++)
			{
				_DBG(" .");
				
				for (tmp=0; tmp<0x2FF; tmp++);
			}
			
			_DBG("\r\n Write Done !!! \r\n");	
			
			cmdn_cnt=0;
		}
		else if (ch_rtn == mod_check[4])		// Flash Verify
		{
			
			result = HAL_CFMC_Verify(0x1F000, size, fbuffer); 
			
			if (result != (0x1F000+size))
			{
				_DBG("Verify Error !! \r\n");
			}
			else
			{
				for(i=0; i<20; i++)
				{
					_DBG(" .");
					
					for (tmp=0; tmp<0x2FF; tmp++);
				}
				
				_DBG("\r\n Verify Done !!! \r\n");
			}
			
			cmdn_cnt=0;
		}	
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*CFMC WriteErase Test start*/
		CFMC_WriteEraseRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
