/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void FRT_Configure(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void FRT_IRQHandler_IT(void);

/* Private variables ---------------------------------------------------------*/
static uint32_t matchintcnt;
static uint32_t g_FRT_Flag;
static uint32_t 	counter;

static const uint8_t menu[] =
"************************************************\n\r"
" A33M11x_TEST_EXAMPLE \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t FRT Match Test Example \n\r"
"************************************************\n\r";

/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      FRT interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void FRT_IRQHandler_IT(void) {
	uint16_t status;
	
	status = HAL_FRT_GetStatus();				// Get FRT Status
	
	if (status & FRT_STAT_MATCHI){ // match interrupt  
		HAL_FRT_ClearStatus(FRT_STAT_MATCHI);
		matchintcnt++;
	}
	
	g_FRT_Flag=1;
}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{

}

/**********************************************************************
 * @brief		FRT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FRT_Configure(void)
{	
	if(HAL_FRT_Init(FRT_CTRL_MODE_MATCH) != HAL_OK)
	{
		Error_Handler();
	}
	
	HAL_FRT_SetMatchCounter(8000000); //1s
	
	/* FRT Clock Source = HSE (8MHz) */
	HAL_FRT_ClockSource(SCU_MCCR_CSEL_HSE, 1);
		
			
	/* FRT Interrupt(Match) Setting */
	HAL_FRT_MatchInterrupt(ENABLE);	// FRT match interrupt
	
	// SET FRT Interrupt Priority
	NVIC_SetPriority(FRT_IRQn, ((0x01<<1)|0x01));	
	// ENABLE FRT Interrupt 	
	NVIC_EnableIRQ(FRT_IRQn);	
	
	HAL_FRT_Run(ENABLE);			// FRT Enable
	HAL_FRT_ClearCounter();		// Clear FRT Counter
	
	counter = 0;
	matchintcnt=0;
	
}


/**********************************************************************
 * @brief		FRT_MatchRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void FRT_MatchRun(void)
{
	if(g_FRT_Flag == 1)
	{
		counter = matchintcnt;		// Get FRT Counter 
		_DBG(" - Count :"); _DBD16(counter); _DBG("\r\n");
		g_FRT_Flag = 0;
	}
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*FRT Configure*/
	FRT_Configure(); 
  
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*FRT match test start*/
		FRT_MatchRun();		
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
