/**********************************************************************
* @file		A33M11x_it.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define USED_CLKO
/*#define USED_LSI */		
/*#define USED_HSI */		
#define USED_HSE	
/*#define USED_PLL96*/ 

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config (void);
/* Private variables ---------------------------------------------------------*/


/**************************************************************************
 * @brief			Initialize default clock for A33M11x Board
 * @param[in]		None
 * @return			None
 **************************************************************************/
void SystemClock_Config (void)
{
	uint32_t i;
	// Flash Wait Config
	HAL_CFMC_WaitCmd(7);
	
	// CLKO Function Setting, Check PORT Setting (PC9)
#ifdef USED_CLKO
	HAL_SCU_ClockOutput(SCU_COR_CLKOINSEL_MCLK, 0x5, ENABLE);			// CLKO=PCLK, Divider=5
#else
	HAL_SCU_ClockOutput(SCU_COR_CLKOINSEL_MCLK, 0x5, DISABLE);			
#endif
	
	//!! For Clock Change, Enable All Clock Source !!
	HAL_SCU_ClockSRCCmd((SCU_CSCR_HSE_ON|SCU_CSCR_HSI_ON|SCU_CSCR_LSI_ON), ENABLE);
	
	for (i=0;i<10;i++);	
	// Main Clock Selection
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_LSI);
	
	// MCLK Monitoring Disable
	HAL_SCU_ClockMonitoringCmd(SCU_CMR_MCLKMNT, DISABLE);
	
#ifdef USED_LSI		//500khz
	
	HAL_SCU_HCLKDIVCmd(0);			// HCLK=MCLK
	HAL_SCU_PCLKDIVCmd(0);			// PCLK=HCLK
	
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_LSI);

	SystemCoreClock = 500000;
	SystemPeriClock = 500000;

	HAL_SCU_MiscClockConfig(7, UART_TYPE, SCU_MCCR_CSEL_LSI, 3);	
#endif

#ifdef USED_HSI		//32Mhz

	HAL_SCU_HCLKDIVCmd(0);			// HCLK=MCLK
	HAL_SCU_PCLKDIVCmd(0);			// PCLK=HCLK
	
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_HSI);
	
	SystemCoreClock = 32000000;
	SystemPeriClock = 32000000;	

	HAL_SCU_MiscClockConfig(7, UART_TYPE, SCU_MCCR_CSEL_HSI, 2);
#endif

#ifdef USED_HSE		//8MHz

	HAL_SCU_HCLKDIVCmd(0);			// HCLK=MCLK
	HAL_SCU_PCLKDIVCmd(0);			// PCLK=HCLK
	
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_HSE);
	
	SystemCoreClock = 8000000;
	SystemPeriClock = 8000000;
	
	HAL_SCU_MiscClockConfig(7, UART_TYPE, SCU_MCCR_CSEL_HSE, 2);
#endif

#ifdef USED_PLL96		// 96MHz, Source HSE
	
	HAL_SCU_HCLKDIVCmd(0);			// HCLK=MCLK
	HAL_SCU_PCLKDIVCmd(0);			// PCLK=HCLK

	// Main Clock Selection - HSE 8MHz
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_HSE);
	
	HAL_SCU_PLLINCLKCmd(SCU_SCCR_PLLCLKSEL_HSE);	// PLL Input Clock = HSE
	
	HAL_SCU_PLLPREDIVCmd(0);		// PLLINCLK = 8MHz
	
	if (HAL_SCU_PLL_ClockConfig(SCU_PLLCON_PLLMODE_FOUT, /*0:FOUT==VCO, 1:FOUT==2xVCO, D=0*/
		7, /*PREDIV, R=7*/
		95, /*POSTDIV1, N1=95*/
		0, 	/*POSTDIV2, N2=0*/
		0,	/*OUTDIV P=0*/
		ENABLE) == ERROR)
	{
		Error_Handler();
	}
	
	HAL_SCU_SystemClockChange(SCU_SCCR_MCLKSEL_PLL);
	
	SystemCoreClock = 96000000;
	SystemPeriClock = 96000000;
	
	HAL_SCU_MiscClockConfig(7, UART_TYPE, SCU_MCCR_CSEL_PLL, 2);
#endif	
}


