/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for MPWM firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void MPWM0PROT_IRQHandler_IT(void);
void MPWM0OVV_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void Delay(uint32_t val);
void mainloop(void);
int main (void);
void Error_Handler(void);
	

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Motor PWM interrupt demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using MPWM\n\r"
"************************************************\n\r";

const uint8_t psel_menu_m_t[] =
"************************************************\n\r"
"Comparator Protection Test (AFE3) \r\n"
"\t - Protection H-activie \n\r"
"\t - PA13 : AIP3, PA14 : AIN3, PA15 : CO3 \n\r"
"\t - If the output value of the comparator is 'H', \n\r"
"\t   the protection function operates\n\r"
"\t - MPWM0 protection interrupt has occurred.. PA2 Toggle\n\r"
"\t - Enables the protection output at the level set in FOLR\n\r"
"\t - PB0 :MP0UH => Output force level :Low\n\r"
"\t - PB1 :MP0UL => Output force level:High\n\r"
"************************************************\n\r";

const uint8_t modsel_m_t[] =
"------------------------------------------------\n\r"
"Motor PWM Mode Selection \n\r"
"\t - 0 : Motor Mode \n\r"
"\t - 1 : PWM Mode \n\r"
"\t - 2 : Extend Mode \n\r"
"------------------------------------------------\n\r";


/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		MPWM0PROT_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0PROT_IRQHandler_IT(void)
{
	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_PROTIF);
	PA->ODR^=(1<<2);
	Delay(500);
}


/**********************************************************************
 * @brief		MPWM0OVV_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0OVV_IRQHandler_IT(void)
{
	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_OVV,MPWM_OSR_OVINIF);
	PA->ODR^=(1<<3);
	Delay(500);
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		Delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Delay(uint32_t val)
{
	uint32_t i;
	for(i=0;i<val;i++);
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Inerrupt Checking Pin
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_PA0);
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_PA1);
	HAL_GPIO_ConfigOutput(PA, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_PA2);
	HAL_GPIO_ConfigOutput(PA, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);
	
	// MPWM pin setting
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_MPWM0UH);
	HAL_GPIO_ConfigureFunction(PB, 1, PB1_MUX_MPWM0UL);
	HAL_GPIO_ConfigureFunction(PB, 2, PB2_MUX_MPWM0VH);
	HAL_GPIO_ConfigureFunction(PB, 3, PB3_MUX_MPWM0VL);
	HAL_GPIO_ConfigureFunction(PB, 4, PB4_MUX_MPWM0WH);
	HAL_GPIO_ConfigureFunction(PB, 5, PB5_MUX_MPWM0WL);
	
	// Protection pin setting
	HAL_GPIO_ConfigureFunction(PB, 6, PB6_MUX_PRTIN0U);
	HAL_GPIO_ConfigureFunction(PB, 7, PB7_MUX_OVIN0U);
	HAL_GPIO_ConfigureFunction(PB, 15, PB15_MUX_OVINEV);
	HAL_GPIO_ConfigureFunction(PB, 14, PB14_MUX_PRTINEV);
	HAL_GPIO_ConfigureFunction(PF, 6, PF6_MUX_PRTINEW);
	HAL_GPIO_ConfigureFunction(PF, 7, PF7_MUX_OVINEW);

	// COMP3 pin setting
	// AIP3(PA13)
	HAL_GPIO_ConfigureFunction(PA, 13, PA13_MUX_AIP3);
	HAL_GPIO_ConfigOutput(PA, 13, INPUT);
	HAL_GPIO_ConfigPullup(PA, 13, PULL_UP_DOWN_DISABLE);
	// AIN3(PA14)
	HAL_GPIO_ConfigureFunction(PA, 14, PA14_MUX_AIN3);
	HAL_GPIO_ConfigOutput(PA, 14, INPUT);
	HAL_GPIO_ConfigPullup(PA, 14, PULL_UP_DOWN_DISABLE);	
	// CO3(PA15)	
	HAL_GPIO_ConfigureFunction(PA,15, PA15_MUX_CO3);
	HAL_GPIO_ConfigOutput(PA, 15, PUSH_PULL_OUTPUT);
	
}

/**********************************************************************
 * @brief		COMP_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void COMP_Configure(void)
{
	HAL_AFE_ClockInit(ENABLE);	// Comparator Peri, Peri Clock Enable

	HAL_AFE_COMP_Cmd(3, ENABLE); //Comparator3 enable
}


/**********************************************************************
 * @brief		MPWM_CompTestRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void MPWM_CompTestRun(void)
{		
	char 		get_md;
	uint32_t 	dutyh, dutyl;
	IRQn_Type	MPWMxPROT_IRQn;
		
	MPWMxPROT_IRQn = MPWM0PROT_IRQn;
			
	_DBG(modsel_m_t);
	_DBG("A33M11x >");
	
	get_md = UARTGetChar(UART0);
	
	_DBC(get_md);
	_DBG("\r\n");
		
	HAL_SCU_MiscClockConfig(2,MPWM0_TYPE,SCU_MCCR_CSEL_MCLK,2); // select clock source, HSE/2, 4MHz
	
	
	// Duty 50% Setting
	dutyh=0x5000;
	dutyl=0x5000;
				
	if (get_md == 0x30)	// MPWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
		HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,ASYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (1-ch asymmetric)

		HAL_MPWM_SetPeriod(MPWM0,0xa000);	// Phase U period setting
		
		// Duty 50% Setting
		HAL_MPWM_SetUDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetVDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetWDuty(MPWM0,dutyh,dutyl);
		
		// DEADTIME & P - SHORT ======>>>> OFF
		HAL_MPWM_SetDeadTime(MPWM0,DTIME_DIS,PSHRT_EN,PWMCLK_2,1,LEAD_EDGE_PWMxH); // dead time setting 		
		
		HAL_MPWM_Start(MPWM0, ENABLE);	// PWM Start

		_DBG("\n\r");
		_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
		
	}
	else if (get_md == 0x31)	// NORMAL PWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
		HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, Not use update timing)

		HAL_MPWM_SetPeriod(MPWM0,0xa000);	// Phase U period setting

		// Duty 50% Setting
		HAL_MPWM_SetUDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetVDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetWDuty(MPWM0,dutyh,dutyl);
		
		// DEADTIME & P - SHORT ======>>>> OFF
		HAL_MPWM_SetDeadTime(MPWM0,DTIME_DIS,PSHRT_EN,PWMCLK_2,1,LEAD_EDGE_PWMxH); // dead time setting 		
		
		HAL_MPWM_Start(MPWM0, ENABLE);	// PWM Start

		_DBG("\n\r");
		_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
	
	}
	else if (get_md == 0x32)	// INDIVIDUAL PWM MODE
	{
		HAL_MPWM_ExtInit(MPWM0);
		
		HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (1-ch symmetric)
		
		// Phase U,V,W period setting
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_U, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_V, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_W, 0xa000);
		
		// Duty 50% Setting
		HAL_MPWM_SetUDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetVDuty(MPWM0,dutyh,dutyl);
		HAL_MPWM_SetWDuty(MPWM0,dutyh,dutyl);
		
		//	DEADTIME & P - SHORT ======>>>> OFF
		/* deadtime not use, pshort use */
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_U, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_V, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_W, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);

		HAL_MPWM_ExtStartCmd(MPWM0,PH_U,MPWM_Ext_START);	// Phase-U Start
		HAL_MPWM_ExtStartCmd(MPWM0,PH_V,MPWM_Ext_START);	// Phase-V Start
		HAL_MPWM_ExtStartCmd(MPWM0,PH_W,MPWM_Ext_START);	// Phase-W Start											
		
		_DBG("\n\r");
		_DBG("MPWM0->CR3="); _DBH(MPWM0->CR3);_DBG("\n\r");
	}
	
	_DBG(psel_menu_m_t);
	
	// Over Voltage, Protection Interrupt
	NVIC_SetPriority(MPWMxPROT_IRQn, 1);
	
	// Enable IRQ
	NVIC_EnableIRQ(MPWMxPROT_IRQn);
	
	//	Protection Interrupt Setting
	HAL_MPWM_ProtIntConfig(MPWM0,PROTSEL_PROT,ENABLE);
	
	__enable_irq();
	
	// Protection Detection output at the L-side of U/V/W Phase
	HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, LOW_SIDE, ENABLE);
	HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, HIGH_SIDE, ENABLE);
	
	// Force output level L-side = High, H-side=Low
	HAL_MPWM_SetFOutput(MPWM0,PH_U,LOW_SIDE,HIGH_LEVEL);
	HAL_MPWM_SetFOutput(MPWM0, PH_U, HIGH_SIDE, LOW_LEVEL);
	
	// Comp3 Select /  Protection Enable
	HAL_MPWM_SetCMPProt(MPWM0, PH_U, 3, ENABLE);

// U Phase Protection H-active, debounce clk=0
	HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_U,PROTPOL_HIGH_ACTIVE,0,ENABLE);
	
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*Configure comp peripheral*/
	COMP_Configure();
	
	 /* Enable IRQ Interrupts */
	__enable_irq();	

	/*MPWM CompTest start */
	MPWM_CompTestRun();
	
	while(1)
	{

	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
