/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void NMI_Handler_IT(void);
void MPWM0PROT_IRQHandler_IT(void);
void MPWM0OVV_IRQHandler_IT(void);
void MPWM0U_IRQHandler_IT(void);
void MPWM0V_IRQHandler_IT(void);
void MPWM0W_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
	

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Motor PWM interrupt demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using MPWM\n\r"
"************************************************\n\r";
const uint8_t psel_menu_m_t[] =
"************************************************\n\r"
"Protection Mode Selection \n\r"
"\t - 0 :  Protection L-activie \n\r"
"\t - 1 :  Protection H-activie \n\r"
"\t - 2 :  Overvoltage L-active \n\r"
"\t - 3 :  Overvoltage H-active \n\r"
"\t - 4 :  Protection Detection output at the L-side of U/V/W Phase \n\r"
"\t - 5 :  Protection Detection output at the H-side of U/V/W Phase \n\r"
"\t - 6 :  Overvoltage Detection output at the L-side of U/V/W Phase \n\r"
"\t - 7 :  Overvoltage Detection output at the H-side of U/V/W Phase \n\r"
"\t - 8 :  Overvoltage Detection output at the both side of U/V/W Phase \n\r"
"************************************************\n\r";

const uint8_t protc_menu_m_t[] =
"************************************************\n\r"
"Protection Status Clear Selection \n\r"
"\t - 0 :  ULPROT \n\r"
"\t - 1 :  VLPROT  \n\r"
"\t - 2 :  WLPROT  \n\r"
"\t - 3 :  UHPROT  \n\r"
"\t - 4 :  VHPROT  \n\r"
"\t - 5 :  WHPROT  \n\r"
"\t - 6 :  PROTIF  \n\r"
"************************************************\n\r";

const uint8_t int_menu_m_t[] =
"************************************************\n\r"
"Interrupt Enable Selection \n\r"
"\t - 0 :  UL Duty or ATR1 Interrupt \n\r"
"\t - 1 :  VL Duty or ATR2 Interrupt \n\r"
"\t - 2 :  WL Duty or ATR3 Interrupt \n\r"
"\t - 3 :  UH Duty or ATR4 Interrupt \n\r"
"\t - 4 :  VH Duty or ATR5 Interrupt \n\r"
"\t - 5 :  WH Duty or ATR6 Interrupt \n\r"
"\t - 6 :  Bottom Interrupt \n\r"
"\t - 7 :  Period Interrupt \n\r"
"************************************************\n\r";
const uint8_t modsel_m_t[] =
"------------------------------------------------\n\r"
"Motor PWM Mode Selection \n\r"
"\t - 0 : Motor Mode \n\r"
"\t - 1 : PWM Mode \n\r"
"\t - 2 : Extend Mode \n\r"
"------------------------------------------------\n\r";

const uint8_t test_menu_m_t[] =
"************************************************\n\r"
"Test Command  \n\r"
"\t 'test adc' : ADC ATR Test Mode \n\r"
"\t 'n' : W/V/U Phase Counter Halt \n\r"
"\t 'm' : W/V/U Phase Counter Continue \n\r"
"\t 'x' : W/V/U Phase Counter Start \n\r"
"\t 't' : W/V/U Phase Counter Stop \n\r"
"\t 'i' : Interrupt Enable Mode \n\r"
"\t 'q' : Protection Status Clear Mode \n\r"
"\t 'p' : Protection Setting Mode \n\r"
"\t ' ' : Protection Setting and Status Check \n\r"
"\t '0' : MPWM Set Delta Duty value to 1\n\r"
"\t '1' : MPWM Set DeltaDuty value to 0x10\n\r"
"\t 's' : MPWM Mode(Motor/Normal/Individual) Change \n\r"
"\t 'u' : MPWM Up/Down Counter Mode Change \n\r"
"\t '[' : xH Phase Counter Duty Change (DutyH-=Delta) \n\r"
"\t ']' : xH Phase Counter Duty Change (DutyH+=Delta) \n\r"
"\t '{' : xL Phase Counter Duty Change (DutyL-=Delta) \n\r"
"\t '}' : xL Phase Counter Duty Change (DutyL+=Delta) \n\r"
"\t ',' : ATR1 Counter Change (trg-=Delta) \n\r"
"\t '.' : ATR1 Counter Change (trg+=Delta) \n\r"
"\t 'd' : Enter \n\r"
"************************************************\n\r";

static	int	_prot0 = 0;
static	int	_prot1 = 0;

uint32_t count_m_t;

uint32_t fflag_m_t;
uint16_t adcval_m_t[8];

/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		MPWM0PROT_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0PROT_IRQHandler_IT(void)
{
	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_PROTIF);
	_prot0++;
}


/**********************************************************************
 * @brief		MPWM0OVV_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0OVV_IRQHandler_IT(void)
{
	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_OVV,MPWM_OSR_OVINIF);
	_prot1++;
}


/**********************************************************************
 * @brief		MPWM0U_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0U_IRQHandler_IT(void)
{
	int	irq;
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDUIF)	//BV_PRDIF_PH_U
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDUIF);
		PA->ODR^=(1<<7);
	}
	if(irq&MPWM_SR_BOTUIF)	//BV_PRDIF_PH_U
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTUIF);
		PA->ODR^=(1<<6);
	}
	if(irq&MPWM_SR_DWHIF_ATR6F)	//BV_WHIF_ATR6F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWHIF_ATR6F);
		PA->ODR^=(1<<5);
	}
	if(irq&MPWM_SR_DVHIF_ATF5F)	//BV_VHIF_ATR5F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVHIF_ATF5F);
		PA->ODR^=(1<<4);
	}
	if(irq&MPWM_SR_DUHIF_ATR4F)	//BV_UHIF_ATR4F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DUHIF_ATR4F);
		PA->ODR^=(1<<3);
	}
	if(irq&MPWM_SR_DWLIF_ATF3F)	//BV_WLIF_ATR3F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWLIF_ATF3F);
		PA->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DVLIF_ATR2F)	//BV_VHIF_ATR2F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVLIF_ATR2F);
		PA->ODR^=(1<<1);
	}
	if(irq&MPWM_SR_DULIF_ATF1F)	//BV_UHIF_ATR1F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DULIF_ATF1F);
		PA->ODR^=(1<<0);
	}
}

/**********************************************************************
 * @brief		MPWM0V_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0V_IRQHandler_IT(void)
{
	int	irq;
	
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDVIF)	//BV_PRDIF_PH_V
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDVIF);
		PA->ODR^=(1<<9);
	}
	if(irq&MPWM_SR_BOTVIF)	//BV_PRDIF_PH_V
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTVIF);
		PA->ODR^=(1<<8);
	}
	if(irq&MPWM_SR_DWHIF_ATR6F)	//BV_WHIF_ATR6F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWHIF_ATR6F);
		PA->ODR^=(1<<5);
	}
	if(irq&MPWM_SR_DVHIF_ATF5F)	//BV_VHIF_ATR5F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVHIF_ATF5F);
		PA->ODR^=(1<<4);
	}
	if(irq&MPWM_SR_DUHIF_ATR4F)	//BV_UHIF_ATR4F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DUHIF_ATR4F);
		PA->ODR^=(1<<3);
	}
	if(irq&MPWM_SR_DWLIF_ATF3F)	//BV_WLIF_ATR3F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWLIF_ATF3F);
		PA->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DVLIF_ATR2F)	//BV_VHIF_ATR2F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVLIF_ATR2F);
		PA->ODR^=(1<<1);
	}
	if(irq&MPWM_SR_DULIF_ATF1F)	//BV_UHIF_ATR1F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DULIF_ATF1F);
		PA->ODR^=(1<<0);
	}
}


/**********************************************************************
 * @brief		MPWM0W_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0W_IRQHandler_IT(void)
{
	int	irq;
	
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDWIF)	//BV_PRDIF_PH_W
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDWIF);
		PA->ODR^=(1<<11);
	}
	if(irq&MPWM_SR_BOTWIF)	//BV_PRDIF_PH_W
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTWIF);
		PA->ODR^=(1<<10);
	}
	if(irq&MPWM_SR_DWHIF_ATR6F)	//BV_WHIF_ATR6F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWHIF_ATR6F);
		PA->ODR^=(1<<5);
	}
	if(irq&MPWM_SR_DVHIF_ATF5F)	//BV_VHIF_ATR5F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVHIF_ATF5F);
		PA->ODR^=(1<<4);
	}
	if(irq&MPWM_SR_DUHIF_ATR4F)	//BV_UHIF_ATR4F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DUHIF_ATR4F);
		PA->ODR^=(1<<3);
	}
	if(irq&MPWM_SR_DWLIF_ATF3F)	//BV_WLIF_ATR3F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWLIF_ATF3F);
		PA->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DVLIF_ATR2F)	//BV_VHIF_ATR2F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVLIF_ATR2F);
		PA->ODR^=(1<<1);
	}
	if(irq&MPWM_SR_DULIF_ATF1F)	//BV_UHIF_ATR1F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DULIF_ATF1F);
		PA->ODR^=(1<<0);
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Inerrupt Checking Pin
	HAL_GPIO_ConfigureFunction(PA, 0, PA0_MUX_PA0);
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 0, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 1, PA1_MUX_PA1);
	HAL_GPIO_ConfigOutput(PA, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 1, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PA, 2, PA2_MUX_PA2);
	HAL_GPIO_ConfigOutput(PA, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 2, PULL_UP_DOWN_DISABLE);
	
	// MPWM pin
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_MPWM0UH);
	HAL_GPIO_ConfigureFunction(PB, 1, PB1_MUX_MPWM0UL);
	HAL_GPIO_ConfigureFunction(PB, 2, PB2_MUX_MPWM0VH);
	HAL_GPIO_ConfigureFunction(PB, 3, PB3_MUX_MPWM0VL);
	HAL_GPIO_ConfigureFunction(PB, 4, PB4_MUX_MPWM0WH);
	HAL_GPIO_ConfigureFunction(PB, 5, PB5_MUX_MPWM0WL);
	HAL_GPIO_ConfigureFunction(PB, 6, PB6_MUX_PRTIN0U);
	HAL_GPIO_ConfigureFunction(PB, 7, PB7_MUX_OVIN0U);
}


/**********************************************************************
 * @brief		check_prot
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void	check_prot(void)
{
	if(_prot0) {
		_DBG("\n\r");
		_DBG("PROT0="); _DBH(_prot0);_DBG("\n\r");		
		_prot0=0;
	}
	if(_prot1) {
		_DBG("\n\r");
		_DBG("PROT1="); _DBH(_prot1);_DBG("\n\r");			
		_prot1=0;
	}
}


/**********************************************************************
 * @brief		MPWM_PWMRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void MPWM_PWMRun(void)
{		
	char 		cmd, ch, get_md;
	uint16_t 	dutyh, dutyl;
	uint16_t 	delta=0x10;
	int			trg, mode;
	
	IRQn_Type	M_MPWMx_IRQn, E_MPWMxU_IRQn, E_MPWMxV_IRQn, E_MPWMxW_IRQn;
	IRQn_Type	MPWMxPROT_IRQn, MPWMxOVV_IRQn;
		
	__enable_irq();
		
	_DBG(modsel_m_t);
	_DBG("A33M11x >");
	
	get_md = UARTGetChar(UART0);
	
	_DBC(get_md);
	_DBG("\r\n");
	
	__disable_irq();
	
	HAL_SCU_MiscClockConfig(2,MPWM0_TYPE,SCU_MCCR_CSEL_MCLK,2); // select clock source, HSE/2, 4MHz

	M_MPWMx_IRQn = MPWM0U_IRQn;
		
	E_MPWMxU_IRQn = MPWM0U_IRQn;
	E_MPWMxV_IRQn = MPWM0V_IRQn;
	E_MPWMxW_IRQn = MPWM0W_IRQn;
		
	MPWMxPROT_IRQn = MPWM0PROT_IRQn;
	MPWMxOVV_IRQn = MPWM0OVV_IRQn;
	if (get_md == 0x30)	// MPWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
//	HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (2-ch symmetric)
		
		HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,ASYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (1-ch asymmetric)
		
//	HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (1-ch symmetric)
	}
	else if (get_md == 0x31)	// NORMAL PWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
//	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_2CH,UP_COUNT);	// MPWM mode reg setting - Normal PWM mode
		
		HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, Not use update timing)
		
//	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, period udate)
		
//	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, bottom udate)

	}
	else if (get_md == 0x32)	// INDIVIDUAL PWM MODE
	{
		HAL_MPWM_ExtInit(MPWM0);
		
//	HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (2-ch symmetric)

//	HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,ASYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (1-ch asymmetric)

		HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (1-ch symmetric)
		
		HAL_GPIO_ConfigureFunction(PB, 14, PB14_MUX_PRTINEV);
		HAL_GPIO_ConfigureFunction(PB, 15, PB15_MUX_OVINEV);
		
		HAL_GPIO_ConfigureFunction(PA, 14, PA14_MUX_PRTINEV);
		HAL_GPIO_ConfigureFunction(PA, 15, PA15_MUX_OVINEV);
	
		HAL_GPIO_ConfigureFunction(PF, 6, PF6_MUX_PRTINEW);
		HAL_GPIO_ConfigureFunction(PF, 7, PF7_MUX_OVINEW);
		
	}
	
	if (get_md == 0x30 || get_md == 0x31)
	{
		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(M_MPWMx_IRQn, 3);
		/* Enable Interrupt for MPWM channel */
		NVIC_EnableIRQ(M_MPWMx_IRQn);
	}
	else if (get_md == 0x32)
	{
		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(E_MPWMxU_IRQn, 3);
		NVIC_SetPriority(E_MPWMxV_IRQn, 3);
		NVIC_SetPriority(E_MPWMxW_IRQn, 3);
		/* Enable Interrupt for MPWM channel */
		NVIC_EnableIRQ(E_MPWMxU_IRQn);
		NVIC_EnableIRQ(E_MPWMxV_IRQn);
		NVIC_EnableIRQ(E_MPWMxW_IRQn);
	}
	
	// Over Voltage, Protection Interrupt
	NVIC_SetPriority(MPWMxPROT_IRQn, 1);
	NVIC_SetPriority(MPWMxOVV_IRQn, 1);
	
	// Enable IRQ
	NVIC_EnableIRQ(MPWMxPROT_IRQn);
	NVIC_EnableIRQ(MPWMxOVV_IRQn);

	//===========================================
	// Duty 50% Setting
	//===========================================
	dutyh=0x5000;
	dutyl=0x5000;
	
	if (get_md == 0x32)	// Phase U,V,W period setting
	{
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_U, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_V, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_W, 0xa000);
	}
	else
	{
		HAL_MPWM_SetPeriod(MPWM0,0xa000);	// Phase U period setting
	}
	
	HAL_MPWM_SetUDuty(MPWM0,dutyh,dutyl);
	HAL_MPWM_SetVDuty(MPWM0,dutyh,dutyl);
	HAL_MPWM_SetWDuty(MPWM0,dutyh,dutyl);
	
	if (get_md == 0x32)
	{	
	//=============================================================
	//			DEADTIME & P - SHORT ======>>>> OFF
	//=============================================================
		/* deadtime not use, pshort use */
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_U, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_V, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_W, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
	}
	else
	{
	//=============================================================
	//			DEADTIME & P - SHORT ======>>>> OFF
	//=============================================================
		HAL_MPWM_SetDeadTime(MPWM0,DTIME_DIS,PSHRT_EN,PWMCLK_2,1,LEAD_EDGE_PWMxH); // dead time setting 		
	}
	
	mode=0;

	__enable_irq();

	_DBG(test_menu_m_t);
	
	while(1){
		_DBG("*");	
		getstring();
    
		if (InFlag) {
			_DBG("\n\r");
			cmd=InData[0];
			
			switch(cmd) {
				case 'n':	//halt : stops the counter clock only
					if (get_md == 0x32)
					{
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_U, MPWM_Ext_HALT);	// Phase-U Halt
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_V, MPWM_Ext_HALT);	// Phase-V Halt
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_W, MPWM_Ext_HALT);	// Phase-W Halt
						
						_DBG("\n\r");
						_DBG("MPWM0->CR3="); _DBH(MPWM0->CR3);_DBG("\n\r");
					}
					else
					{
						HAL_MPWM_Halt(MPWM0,ENABLE);
						_DBG("\n\r");
						_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
					}
					break;
				case 'm':	//continue : resume phase counter
					if (get_md == 0x32)
					{
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_U, MPWM_Ext_CONT);	// Phase-U Cont
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_V, MPWM_Ext_CONT);	// Phase-V Cont
						HAL_MPWM_ExtPauseCmd(MPWM0, PH_W, MPWM_Ext_CONT);	// Phase-W Cont
						
					}
					break;
				case 'x':	//start : phase start
					if (get_md == 0x32)
					{
						PA->ODR = 0x0;
						
						HAL_MPWM_ExtStartCmd(MPWM0,PH_U,MPWM_Ext_START);	// Phase-U Start
						HAL_MPWM_ExtStartCmd(MPWM0,PH_V,MPWM_Ext_START);	// Phase-V Start
						HAL_MPWM_ExtStartCmd(MPWM0,PH_W,MPWM_Ext_START);	// Phase-W Start											
						
						HAL_MPWM_ExtCaptureCmd(MPWM0,PH_U,SUBCAP_RISING,ENABLE);
						HAL_MPWM_ExtCaptureCmd(MPWM0,PH_U,SUBCAP_RISING,DISABLE);
						HAL_MPWM_ExtCaptureCmd(MPWM0,PH_U,SUBCAP_FALLING,ENABLE);
						HAL_MPWM_ExtCaptureCmd(MPWM0,PH_U,SUBCAP_FALLING,DISABLE);
						
						_DBG("\n\r");
						_DBG("MPWM0->CR3="); _DBH(MPWM0->CR3);_DBG("\n\r");
						
					}
					else
					{
						PA->ODR = 0x0;
						HAL_MPWM_Start(MPWM0, ENABLE);	// PWM Start

						_DBG("\n\r");
						_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
					}
					break;	
				case 't':	//stop : phase stop
					if (get_md == 0x32)
					{
						PA->ODR = 0x0;
						
						HAL_MPWM_ExtStartCmd(MPWM0,PH_U,MPWM_Ext_STOP);	// Phase-U Stop
						HAL_MPWM_ExtStartCmd(MPWM0,PH_V,MPWM_Ext_STOP);	// Phase-V Stop
						HAL_MPWM_ExtStartCmd(MPWM0,PH_W,MPWM_Ext_STOP);	// Phase-W Stop

						_DBG("\n\r");
						_DBG("MPWM0->CR3="); _DBH(MPWM0->CR3);_DBG("\n\r");
						
					}
					else
					{
						PA->ODR = 0x0;
						
						HAL_MPWM_Start(MPWM0, DISABLE);	// PWM STOP

						_DBG("\n\r");
						_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
					}
					break;
				case 'p':	//protection mode select
					_DBG(psel_menu_m_t);
					ch=_DG;
					_DBC(ch);
					_DBG("\n\r");
					if (get_md == 0x32)
					{
						switch(ch){
						case '0':	// U/V/W Phase Protection L-actve
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_U,PROTPOL_LOW_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_V,PROTPOL_LOW_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_W,PROTPOL_LOW_ACTIVE,0,ENABLE);	
						break; 
						case '1':	// U/V/W Phase Protection H-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_U,PROTPOL_HIGH_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_V,PROTPOL_HIGH_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_W,PROTPOL_HIGH_ACTIVE,0,ENABLE);	
						break; 
						case '2':	// U/V/W Phase Overvoltage L-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_U,PROTPOL_LOW_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_V,PROTPOL_LOW_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_W,PROTPOL_LOW_ACTIVE,0,ENABLE);	
						break; 
						case '3':	// U/V/W Phase Overvoltage H-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_U,PROTPOL_HIGH_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_V,PROTPOL_HIGH_ACTIVE,0,ENABLE);
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_W,PROTPOL_HIGH_ACTIVE,0,ENABLE);	
						break; 
						case '4':	// Protection Detection output at the L-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, LOW_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_V, LOW_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_W, LOW_SIDE, ENABLE);	
						break; 
						case '5':	// Protection Detection output at the H-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, HIGH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_V, HIGH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_W, HIGH_SIDE, ENABLE);		
						break; 
						case '6':	// Overvoltage Detection output at the L-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, LOW_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_V, LOW_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_W, LOW_SIDE, ENABLE);	
						break; 
						case '7':	// Overvoltage Detection output at the H-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, HIGH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_V, HIGH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_W, HIGH_SIDE, ENABLE);	
						break; 
						case '8':	// Overvoltage Detection output at the both side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, BOTH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_V, BOTH_SIDE, ENABLE);
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_W, BOTH_SIDE, ENABLE);	
						break; 
						
						}							
					}
					else
					{
						switch(ch) {
						case '0':	// U Phase Protection L-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_U,PROTPOL_LOW_ACTIVE,0,ENABLE);	
						break; 
						case '1':	// U Phase Protection H-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_PROT, PH_U,PROTPOL_HIGH_ACTIVE,0,ENABLE);	
						break; 
						case '2':	// U Phase Overvoltage L-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_U,PROTPOL_LOW_ACTIVE,0,ENABLE);	
						break; 
						case '3':	// U Phase Overvoltage H-active
							HAL_MPWM_SetProtCtrl(MPWM0,PROTSEL_OVV, PH_U,PROTPOL_HIGH_ACTIVE,0,ENABLE);	
						break; 
						case '4':	// Protection Detection output at the L-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, LOW_SIDE, ENABLE);
						break; 
						case '5':	// Protection Detection output at the H-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_PROT, PH_U, HIGH_SIDE, ENABLE);		
						break; 
						case '6':	// Overvoltage Detection output at the L-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, LOW_SIDE, ENABLE);
						break; 
						case '7':	// Overvoltage Detection output at the H-side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, HIGH_SIDE, ENABLE);
						break; 
						case '8':	// Overvoltage Detection output at the both side of U/V/W Phase
							HAL_MPWM_SetProtOutput(MPWM0, PROTSEL_OVV, PH_U, BOTH_SIDE, ENABLE);
						break; 
							}
					}			
					break;	
				case 'i':	// interrupt mode selection
					_DBG(int_menu_m_t);
					ch=_DG;
					_DBC(ch);
					_DBG("\n\r");
					switch(ch) {
						case '0':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_ULIE_ATR1IE,ENABLE);	break; //BV_ULIE
						case '1':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_VLIE_ATR2IE,ENABLE);	break; //BV_VLIE
						case '2':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_WLIE_ATR3IE,ENABLE);	break; //BV_WLIE
						case '3':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_UHIE_ATR4IE,ENABLE);	break; //BV_UHIE
						case '4':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_VHIE_ATR5IE,ENABLE);	break; //BV_VHIE
						case '5':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_WHIE_ATR6IE,ENABLE);	break; //BV_WHIE
						case '6':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_BOTUIE,ENABLE);	break; //BV_BOTIE
						case '7':	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_PRDUIE,ENABLE);	break; //BV_PRDIE
					}
					_DBG("\n\r");
					_DBG("MPWM0->IER="); _DBH(MPWM0->IER);_DBG("\n\r");						
					break;
				case 'q':	//protection status clear	
					_DBG(protc_menu_m_t);
					ch=_DG;
					switch(ch) {
					case '0': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_ULPROT); break;
					case '1': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_VLPROT); break;
					case '2': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_WLPROT);	break;
					case '3': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_UHPROT);	break;
					case '4': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_VHPROT);	break;
					case '5': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_WHPROT); break;
					case '6': 	HAL_MPWM_ProtStatClear(MPWM0,PROTSEL_PROT,MPWM_PSR_PROTIF); break;
					}
					break;
				case ' ':	// protection status check
					_DBG("\n\r");
					_DBG("PB->IDR="); _DBH(PB->IDR);	
					_DBG("\n\r");
					_DBG("MPWM0->PCR="); _DBH(MPWM0->PCR);	
					_DBG("\n\r");
					_DBG("MPWM0->PSR="); _DBH(MPWM0->PSR);_DBG("\n\r");							
					break;
				case '1':	delta=1; break;
				case '0':	delta=0x10; break;
				case 's':	if(++mode>3) mode=0;
					MPWM0->MR = (MPWM0->MR&~(3<<1))|(mode<<1);
					_DBG("\n\r");
					_DBG("MCHMOD="); _DBD(mode);_DBG("\n\r");					
					break;

				case 'u':	if(MPWM0->MR&0x0001) {	// up/down counter mode selection
								MPWM0->MR&=~0x0001;
								_DBG("(Up-count Mode)");_DBG("\n\r");
							}
							else {
								MPWM0->MR|= 0x0001;
								_DBG("(Up-down Mode)");_DBG("\n\r");
							}
							break;						
				case '[':	dutyh-=delta;	// xH Phase counter duty change (dutyh-=delta)
					MPWM0->DUH= dutyh;
					MPWM0->DVH= dutyh;
					MPWM0->DWH= dutyh;		
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" dutyh="); _DBD16(dutyh);	_DBG("\n\r");	
					break;
				case ']':	dutyh+=delta;	// xH Phase counter duty change (dutyh+=delta)
					MPWM0->DUH= dutyh;
					MPWM0->DVH= dutyh;
					MPWM0->DWH= dutyh;
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" dutyh="); _DBD16(dutyh);_DBG("\n\r");							
					break;
				case '{':	dutyl-=delta;	// xL Phase counter duty change (dutyl-=delta)
					MPWM0->DUL= dutyl;
					MPWM0->DVL= dutyl;
					MPWM0->DWL= dutyl;
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" dutyl="); _DBD16(dutyl);_DBG("\n\r");							
					break;
				case '}':	dutyl+=delta;	// xL Phase counter duty change (dutyl+=delta)
					MPWM0->DUL= dutyl;
					MPWM0->DVL= dutyl;
					MPWM0->DWL= dutyl;
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" dutyl="); _DBD16(dutyl);_DBG("\n\r");					
					break;
				case ',':	trg-= delta;	// ATR1 counter change (trg-=delta)
					MPWM0->ATR1=trg;
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" trg="); _DBD16(trg);_DBG("\n\r");							
					break;
				case '.':	trg+= delta;	// ATR1 counter change (trg+=delta)
					MPWM0->ATR1=trg;
					_DBG("\n\r");
					_DBG("MPWM0->MR="); _DBH(MPWM0->MR);	
					_DBG(" trg="); _DBD16(trg);_DBG("\n\r");							
					break;						
				case 'd':
					_DBG("\n\r");							
					break;
			}
		}			
		else {
			_DBG("\n\r");
			InFlag=0;
			InCount=0;	
		}
	}			
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	 /* Enable IRQ Interrupts */
	__enable_irq();	

	/*MPWM start */
	MPWM_PWMRun();
	
	while(1)
	{

	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
