/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void MPWM0U_IRQHandler_IT(void);
void MPWM0V_IRQHandler_IT(void);
void MPWM0W_IRQHandler_IT(void);
void ADC0_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
	

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"Motor PWM interrupt demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using MPWM\n\r"
"************************************************\n\r";

const uint8_t modsel_m_t[] =
"------------------------------------------------\n\r"
"Motor PWM Mode Selection \n\r"
"\t - 0 : Motor Mode \n\r"
"\t - 1 : PWM Mode \n\r"
"\t - 2 : Extend Mode \n\r"
"------------------------------------------------\n\r";

uint8_t	flag;
uint16_t adcval[8];

/* Public Function ------------------------------------------------------------*/
/**********************************************************************
 * @brief		MPWM0U_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0U_IRQHandler_IT(void)
{
	int	irq;
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDUIF)	//BV_PRDIF_PH_U
	{
		PA->ODR^=(1<<0);
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDUIF);
	}
	if(irq&MPWM_SR_BOTUIF)	//BV_PRDIF_PH_U
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTUIF);
	}
	if(irq&MPWM_SR_DVLIF_ATR2F)	//BV_VHIF_ATR2F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVLIF_ATR2F);
		PC->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DULIF_ATF1F)	//BV_UHIF_ATR1F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DULIF_ATF1F);
		PC->ODR^=(1<<2);
	}
}

/**********************************************************************
 * @brief		MPWM0V_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0V_IRQHandler_IT(void)
{
	int	irq;
	
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDVIF)	//BV_PRDIF_PH_V
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDVIF);
	}
	if(irq&MPWM_SR_BOTVIF)	//BV_PRDIF_PH_V
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTVIF);
	}
	if(irq&MPWM_SR_DUHIF_ATR4F)	//BV_UHIF_ATR4F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DUHIF_ATR4F);
		PC->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DWLIF_ATF3F)	//BV_WLIF_ATR3F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWLIF_ATF3F);
		PC->ODR^=(1<<2);
	}
}


/**********************************************************************
 * @brief		MPWM0W_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void	MPWM0W_IRQHandler_IT(void)
{
	int	irq;
	
	irq = HAL_MPWM_GetIntStat(MPWM0);
	
	if(irq&MPWM_SR_PRDWIF)	//BV_PRDIF_PH_W
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_PRDWIF);
	}
	if(irq&MPWM_SR_BOTWIF)	//BV_PRDIF_PH_W
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_BOTWIF);
	}
	if(irq&MPWM_SR_DWHIF_ATR6F)	//BV_WHIF_ATR6F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DWHIF_ATR6F);
		PC->ODR^=(1<<2);
	}
	if(irq&MPWM_SR_DVHIF_ATF5F)	//BV_VHIF_ATR5F
	{
		HAL_MPWM_IntStatClear(MPWM0,MPWM_SR_DVHIF_ATF5F);		
		PC->ODR^=(1<<2);
	}
}

/**********************************************************************
 * @brief		ADC0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC0_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC0);

	if ((status & ADC_STAT_SEQ)==ADC_STAT_SEQ){
		PC->ODR^=(1<<3);
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_SEQ);
		flag=1;
	}
	
	if ((status & ADC_STAT_SINGLE)==ADC_STAT_SINGLE){
		HAL_ADC_ClearStatus(ADC0, ADC_STAT_SINGLE);		
	}	
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Inerrupt Checking Pin
	HAL_GPIO_ConfigureFunction(PC, 2, PC2_MUX_PC2);
	HAL_GPIO_ConfigOutput(PC, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 2, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PC, 3, PC3_MUX_PC3);
	HAL_GPIO_ConfigOutput(PC, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 3, PULL_UP_DOWN_DISABLE);
	
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_MPWM0UH);
	HAL_GPIO_ConfigureFunction(PB, 1, PB1_MUX_MPWM0UL);
	HAL_GPIO_ConfigureFunction(PB, 2, PB2_MUX_MPWM0VH);
	HAL_GPIO_ConfigureFunction(PB, 3, PB3_MUX_MPWM0VL);
	HAL_GPIO_ConfigureFunction(PB, 4, PB4_MUX_MPWM0WH);
	HAL_GPIO_ConfigureFunction(PB, 5, PB5_MUX_MPWM0WL);
	HAL_GPIO_ConfigureFunction(PB, 6, PB6_MUX_PRTIN0U);
	HAL_GPIO_ConfigureFunction(PB, 7, PB7_MUX_OVIN0U);
}


/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
//ADC_SINGLE_MODE
	ADC_CFG_Type AD0_config;

	AD0_config.Mode = ADC_SINGLE_MODE;
	AD0_config.SamplingTime=0x1f;
	AD0_config.SeqCnt = 6;
	AD0_config.RestartEn = 1;
	AD0_config.TrgSel = ADC_TRIGGER_MPWM0;
	AD0_config.UseClk = ADC_EXTERNAL_CLK;
	AD0_config.InClkDiv = 1;

	HAL_SCU_MiscClockConfig(4, ADC_TYPE, SCU_MCCR_CSEL_MCLK, 8);
	
	if( HAL_ADC_Init(ADC0, &AD0_config) != HAL_OK)
	{
		/* Initialization Error */
     Error_Handler();
	}	
	
	HAL_ADC_TriggerlSel(ADC0, 0x00543210); //use ATR1, ATR2, ATR3, ATR4, ATR5, ATR6

	HAL_ADC_ChannelSel_1(ADC0, 0x01000100);	// ch0 : AN0, ch1:AN1, ch2:AN0, ch3:AN1
	HAL_ADC_ChannelSel_2(ADC0, 0x00000100);	// ch4 :AN0, ch5 : AN1
	
	HAL_ADC_ClearStatus(ADC0, 0x0f); // before adc interrupt enable, you must clear adc status!! 
	HAL_ADC_ConfigInterrupt(ADC0, (ADC_INTEN_SINGLE | ADC_INTEN_SEQ), ENABLE);

	NVIC_SetPriority(ADC0_IRQn, 3);
	/* Enable Interrupt for ADC channel */
  	NVIC_EnableIRQ(ADC0_IRQn);	
}


/**********************************************************************
 * @brief		MPWM_TriggerTestRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void MPWM_TriggerTestRun(void)
{		
	char 		get_md;
	uint16_t 	dutyh, dutyl;
	
	IRQn_Type	M_MPWMx_IRQn, E_MPWMxU_IRQn, E_MPWMxV_IRQn, E_MPWMxW_IRQn;
		
	__enable_irq();
		
	_DBG(modsel_m_t);
	_DBG("A33M11x >");
	
	get_md = UARTGetChar(UART0);
	
	_DBC(get_md);
	_DBG("\r\n");
	
	__disable_irq();
	
	HAL_SCU_MiscClockConfig(2,MPWM0_TYPE,SCU_MCCR_CSEL_MCLK,2); // select clock source, HSE/2, 4MHz

	M_MPWMx_IRQn = MPWM0U_IRQn;
		
	E_MPWMxU_IRQn = MPWM0U_IRQn;
	E_MPWMxV_IRQn = MPWM0V_IRQn;
	E_MPWMxW_IRQn = MPWM0W_IRQn;
		
	if (get_md == 0x30)	// MPWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
	HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (2-ch symmetric)
		
//		HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,ASYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (1-ch asymmetric)
		
//	HAL_MPWM_Cmd(MPWM0,MOTOR_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - MPWM mode (1-ch symmetric)
	}
	else if (get_md == 0x31)	// NORMAL PWM MODE
	{
		HAL_MPWM_Init(MPWM0);
		
//	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_2CH,UP_COUNT);	// MPWM mode reg setting - Normal PWM mode
		
//		HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, Not use update timing)
		
//	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PRD_MAT,UPDATE_PERIOD,UPDATE_DISABLE,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, period udate)
		
	HAL_MPWM_Cmd(MPWM0,NORMAL_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Normal PWM mode (up/down mode, bottom udate)

	}
	else if (get_md == 0x32)	// INDIVIDUAL PWM MODE
	{
		HAL_MPWM_ExtInit(MPWM0);
		
	HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,SYMMETRIC_2CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (2-ch symmetric)

//	HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PRD_MAT,UPDATE_DISABLE,UPDATE_BOTTOM,ASYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (1-ch asymmetric)

//		HAL_MPWM_Cmd(MPWM0,INDIVIDUAL_PWM,PWM_CLOCK,UPDATE_DISABLE,UPDATE_DISABLE,SYMMETRIC_1CH,UP_DOWN_COUNT);	// MPWM mode reg setting - Individual mode (1-ch symmetric)
		
		HAL_GPIO_ConfigureFunction(PB, 14, PB14_MUX_PRTINEV);
		HAL_GPIO_ConfigureFunction(PB, 15, PB15_MUX_OVINEV);
		
		HAL_GPIO_ConfigureFunction(PA, 14, PA14_MUX_PRTINEV);
		HAL_GPIO_ConfigureFunction(PA, 15, PA15_MUX_OVINEV);
	
		HAL_GPIO_ConfigureFunction(PF, 6, PF6_MUX_PRTINEW);
		HAL_GPIO_ConfigureFunction(PF, 7, PF7_MUX_OVINEW);
		
	}
	
	if (get_md == 0x30 || get_md == 0x31)
	{
		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(M_MPWMx_IRQn, 3);
		/* Enable Interrupt for MPWM channel */
		NVIC_EnableIRQ(M_MPWMx_IRQn);
	}
	else if (get_md == 0x32)
	{
		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(E_MPWMxU_IRQn, 3);
		NVIC_SetPriority(E_MPWMxV_IRQn, 3);
		NVIC_SetPriority(E_MPWMxW_IRQn, 3);
		/* Enable Interrupt for MPWM channel */
		NVIC_EnableIRQ(E_MPWMxU_IRQn);
		NVIC_EnableIRQ(E_MPWMxV_IRQn);
		NVIC_EnableIRQ(E_MPWMxW_IRQn);
	}

	//===========================================
	// Duty 50% Setting
	//===========================================
	dutyh=0x5000;
	dutyl=0x5000;
	
	if (get_md == 0x32)	// Phase U,V,W period setting
	{
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_U, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_V, 0xa000);
		HAL_MPWM_ExtSetPeriod(MPWM0, PH_W, 0xa000);
	}
	else
	{
		HAL_MPWM_SetPeriod(MPWM0,0xa000);	// Phase U period setting
	}
	
	HAL_MPWM_SetUDuty(MPWM0,dutyh,dutyl);
	HAL_MPWM_SetVDuty(MPWM0,dutyh,dutyl);
	HAL_MPWM_SetWDuty(MPWM0,dutyh,dutyl);
	
	if (get_md == 0x32)
	{	
	//=============================================================
	//			DEADTIME & P - SHORT ======>>>> OFF
	//=============================================================
		/* deadtime not use, pshort use */
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_U, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_V, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
		HAL_MPWM_ExtSetDeadTime(MPWM0, PH_W, DTIME_DIS, PSHRT_EN, PWMCLK_2, 1, 1, LEAD_EDGE_PWMxH);
	}
	else
	{
	//=============================================================
	//			DEADTIME & P - SHORT ======>>>> OFF
	//=============================================================
		HAL_MPWM_SetDeadTime(MPWM0,DTIME_DIS,PSHRT_EN,PWMCLK_2,1,LEAD_EDGE_PWMxH); // dead time setting 		
	}
	
	__enable_irq();

	_DBG("************************************************\n\r");
	_DBG("\n\rADC ATR test \n\r");
	_DBG("\n\rATR1 : 0x3000, ATR2 : 0x5000, ATR3 : 0x8900 - up counter \n\r");						
	_DBG("\n\rATR4 : 0x8900, ATR5 : 0x5000, ATR6 : 0x3000 - down counter \n\r");			
	_DBG("\n\rPC2 : ATR Interrupt Check, PC3 : ADC Sequence Interrupt Check \n\r");		
	_DBG("************************************************\n\r");
	
	//=============================================================
	//			ATR Use Mode (up&down counter)
	//=============================================================
	HAL_MPWM_SetATR(MPWM0,PH_U,1,TRG_PWM_CLOCK,TRG_UP_COUNT,0x3000);
	HAL_MPWM_SetATR(MPWM0,PH_U,2,TRG_PWM_CLOCK,TRG_UP_COUNT,0x5000);
	HAL_MPWM_SetATR(MPWM0,PH_V,3,TRG_PWM_CLOCK,TRG_UP_COUNT,0x8900);
	HAL_MPWM_SetATR(MPWM0,PH_V,4,TRG_PWM_CLOCK,TRG_DOWN_COUNT,0x8900);
	HAL_MPWM_SetATR(MPWM0,PH_W,5,TRG_PWM_CLOCK,TRG_DOWN_COUNT,0x5000);
	HAL_MPWM_SetATR(MPWM0,PH_W,6,TRG_PWM_CLOCK,TRG_DOWN_COUNT,0x3000);

	/* ATR Interrupt Enable */
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_ULIE_ATR1IE,ENABLE);	// ATR1 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_VLIE_ATR2IE,ENABLE);	// ATR2 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_WLIE_ATR3IE,ENABLE);	// ATR3 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_UHIE_ATR4IE,ENABLE);	// ATR4 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_VHIE_ATR5IE,ENABLE);	// ATR5 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_WHIE_ATR6IE,ENABLE);	// ATR6 Interrupt Enable
	HAL_MPWM_IntConfig(MPWM0,MPWM_IER_PRDUIE,ENABLE);	// Phase-U Period Interrupt Enable
	
	if (get_md == 0x32)
	{
		HAL_MPWM_ExtStartCmd(MPWM0,PH_U,MPWM_Ext_START);	// Phase-U Start
		HAL_MPWM_ExtStartCmd(MPWM0,PH_V,MPWM_Ext_START);	// Phase-V Start
		HAL_MPWM_ExtStartCmd(MPWM0,PH_W,MPWM_Ext_START);	// Phase-W Start											
								
		_DBG("\n\r");
		_DBG("MPWM0->CR3="); _DBH(MPWM0->CR3);_DBG("\n\r");
		
	}
	else
	{
		HAL_MPWM_Start(MPWM0, ENABLE);	// PWM Start

		_DBG("\n\r");
		_DBG("MPWM0->CR2="); _DBH(MPWM0->CR2);_DBG("\n\r");
	}		
	
	while(1)
	{
		while(flag==0);
		flag=0;
//		_DBG("DR0 = "); _DBG("0x"); _DBH16(ADC0->DR0); _DBG("\t");
//		_DBG("DR1 = "); _DBG("0x"); _DBH16(ADC0->DR1); _DBG("\t");
//		_DBG("DR2 = "); _DBG("0x"); _DBH16(ADC0->DR2); _DBG("\r\n");
//		_DBG("DR3 = "); _DBG("0x"); _DBH16(ADC0->DR3); _DBG("\t");
//		_DBG("DR4 = "); _DBG("0x"); _DBH16(ADC0->DR4); _DBG("\t");
//		_DBG("DR5 = "); _DBG("0x"); _DBH16(ADC0->DR5); _DBG("\r\n\n");
		
	}
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();

	/*Configure port peripheral*/
	GPIO_Configure();

	/*Configure ADC peripheral*/	
	ADC_Configure();
	
	 /* Enable IRQ Interrupts */
	__enable_irq();	

	/*MPWM Trigger Test start */
	MPWM_TriggerTestRun();
	
	while(1)
	{

	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
