/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void LVI_Configure(void);
void PWR_LviRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void LVI_IRQHandler_IT(void);

/* Private variables ---------------------------------------------------------*/

static const uint8_t menu[] =
"************************************************\n\r"
" LVI demo \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"\t LVI Test Example \n\r"
"\t LVI Interrupt Mode Level(3.04V) Mode !! \n\r"
"************************************************\n\r";
	
static	LVI_CFG_Type	LVI_config;
static	LVR_CFG_Type	LVR_config;
static uint32_t i;
	
/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      LVI interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void LVI_IRQHandler_IT(void)
{
	_DBG("LVI Interrupt Done !!\r\n");
	
	while(HAL_SCU_GetLVIIntStatus()==SET);	//LVI interrupt status check
	
	HAL_SCU_ClearLVIIntTFlg();	//LVI Flag Clear

}

/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{

}

/**********************************************************************
 * @brief		LVI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void LVI_Configure(void)
{	
	// LVR Configuration - LVR DIsable
	LVR_config.EnSel = DISABLE;
	HAL_LVR_Init(&LVR_config);
	
	// LVI Configuration - LVI Enable, LVI Interrupt Enable, Deepsleep mode on, LVI Level = 3.04V
	LVI_config.EnSel = ENABLE;
	LVI_config.IntSel = ENABLE;
	LVI_config.DeepSel = DISABLE;
	LVI_config.LvlSel = LVI_LVL_3_04V;
	HAL_LVI_Init(&LVI_config);
	
	// Enable Interrupt for LVI
	NVIC_SetPriority(LVI_IRQn, 3);
	NVIC_EnableIRQ(LVI_IRQn);
}

/**********************************************************************
 * @brief		PWR_LviRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_LviRun(void)
{	
			_DBG(" .");	// waiting for LVI interrupt
					
			for (i=0; i<0x0FFFF; i++);
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();

	/*LVI Configure*/
	LVI_Configure(); 
  
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	while(1)
	{
		/*PWR LVI Test start*/
		PWR_LviRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
