/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library
* @version	1.00
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SYSTICK_Configure(void);
void PWR_SleepWakeupSystickRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);
void SysTick_Handler_IT(void);

/* Private variables ---------------------------------------------------------*/
static const uint8_t menu[] =
"************************************************\n\r"
" Example of wake-up using Systick in sleep mode \n\r"
"\t - MCU: A33M11x \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
"************************************************\n\r";
static uint32_t i;
static uint32_t MilliSec;
	
/* Public Function ------------------------------------------------------------*/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_IT(void) {
	if (MilliSec) {
		MilliSec--;
	}
}


/**********************************************************************
 * @brief		 menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		PCU_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB0
	HAL_GPIO_ConfigureFunction(PB, 0, PB0_MUX_PB0);
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, PULL_UP_DOWN_DISABLE);	
	HAL_GPIO_EXTI_ClearPin(PB, _BIT(0));
}



/**********************************************************************
 * @brief		SYSTICK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SYSTICK_Configure(void)
{	
	//1msec interrupt 
	SysTick_Config(SystemCoreClock/1000);  
}

/**********************************************************************
 * @brief		PWR_SleepWakeupSystickRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SleepWakeupSystickRun(void)
{	
			
	_DBG("\n\rEnter Sleep... !\r\n");
	
	// Enter sleep mode
	HAL_PWR_EnterSleepMode();
	
	//clock stability time
	MilliSec = 500;
	while(MilliSec);
	
	for(i=1;i<=5;i++)
	{
		_DBG("count : "); _DBD(i); _DBG("\r\n");
		MilliSec = 500;
		while(MilliSec);
	}
		
	_DBG("Waked Up from sleep mode!");
		
	while(1)
	{
		_DBG(" .");
		PB->ODR ^= 0x01;	
		for (i=0; i<0x0FFFF; i++);
	}
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
 
	/*Configure port peripheral*/
	GPIO_Configure();
 
	//SYSTICK Configure
	SYSTICK_Configure();
 	
	/* Enable IRQ Interrupts */
	__enable_irq();

	/*PWR sleep wake-up Systick test start*/
	PWR_SleepWakeupSystickRun();
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSE 8 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

/* --------------------------------- End Of File ------------------------------ */
