/**********************************************************************
* @file		A33M11x_conf.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Port_Init(void);
/* Private variables ---------------------------------------------------------*/




/**********************************************************************
 * @brief		This function handles NMI exception.
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Port_Init(void)
{
	SYST_ACCESS_EN();
	
	// Peripheral Enable Register1	0 : Disable,	1 : Enable
	SCU->PER1 = SCU->PER1
	| (1<<14)						// GPIOG
	| (1<<13)						// GPIOF
	| (1<<11)						// GPIOD
	| (1<<10)						// GPIOC
	| (1<<9)						// GPIOB
	| (1<<8)						// GPIOA
	;
	
	// Peripheral Clock Enable Register1	0 : Disable,	1 : Enable
	SCU->PCER1 = SCU->PCER1
	| (1<<14)						// GPIOG
	| (1<<13)						// GPIOF
	| (1<<11)						// GPIOD
	| (1<<10)						// GPIOC
	| (1<<9)						// GPIOB
	| (1<<8)						// GPIOA
	;
	
	SYST_ACCESS_DIS();
	
	// Enable Writing Permittion of ALL PCU Register
	PORT_ACCESS_EN();
	
	//------------------------------------------------------------
	//	PORT - A
	//------------------------------------------------------------
	
	// PinMux Setting
	PA->MR2 = 0
	| (0<<28)				// 0:PA15,		1:MISO0,		2:QEI0_IDX,		3:OVINEV,			4:CO3,		7:AN15/AO3
	| (0<<24)				// 0:PA14,		1:MOSI0,		2:QEI0_B,			3:PRTINEV,		4:			7:AN14/AIN3
	| (0<<20)				// 0:PA13,		1:SCK0,		2:QEI0_A,			3:				4:			7:AN13/AIP3
	| (0<<16)				// 0:PA12,		1:SS0,		2:				3:,				4:			7:AN12
	| (0<<12)				// 0:PA11,		1:			2:				3:,				4:			7:AN11
	| (0<<8)				// 0:PA10,		1:			2:QEI0_UPDN,		3:SCAPEW,		4:			7:AN10/AIP2
	| (0<<4)				// 0:PA9,			1:			2:				3:SCAPEV,		4:			7:AN9/AIN2
	| (0<<0)				// 0:PA8,			1:			2:				3:SCAPEU,		4:CO2,		7:AN8/AO2
	;
	
	PA->MR1 = 0
	| (0<<28)				// 0:PA7,			1:			2:T3IO,			3:CAPEW,			4:CO1,		7:AN7/AO1
	| (0<<24)				// 0:PA6,			1:			2:T2IO,			3:CAPEV,			4:			7:AN6/AIN1
	| (0<<20)				// 0:PA5,			1:			2:T1IO,			3:,				4:			7:AN5
	| (0<<16)				// 0:PA4,			1:			2:T0IO,			3:				4:			7:AN4
	| (0<<12)				// 0:PA3,			1:			2:				3:CAPEU,			4:			7:AN3/AIP1
	| (0<<8)				// 0:PA2,			1:			2:				3:				4:			7:AN2/AIP0
	| (0<<4)				// 0:PA1,			1:			2:				3:				4:			7:AN1/AIN0
	| (0<<0)				// 0:PA0,			1:			2:				3:				4:CO0,		7:AN0/AO0
	;
	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO)
	PA->CR = 0
	| (0<<30)   // P15 		0:Push-pull output, 1:Open-drain output, 2,3:Input
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	// Pullup/down Setting
	PA->PRCR = 0
	| (0<<30)   // P15 		0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	

	//------------------------------------------------------------
	//	PORT - B
	//------------------------------------------------------------
	// PinMux Setting
	PB->MR2 = 0
	| (0<<28)				// 0:PB15,		1:			2:OVINEV			3:MP1WL,			4:,			7:
	| (0<<24)				// 0:PB14,		1:			2:PRTINEV		3:MP1WH,		4:			7:
	| (0<<20)				// 0:PB13,		1:RXD2		2:MISO1			3:MP1VL,			4:			7:
	| (0<<16)				// 0:PB12,		1:TXD2		2:MOSI1			3:MP1VH,			4:			7:
	| (0<<12)				// 0:PB11,		1:			2:SCK1			3:MP1UL,			4:			7:
	| (0<<8)				// 0:PB10,		1:			2:SS1			3:MP1UH,			4:			7:
	| (0<<4)				// 0:PB9,			1:TXD3,		2:				3:OVIN1U,			4:SWDIO		7:
	| (0<<0)				// 0:PB8,			1:RXD3,		2:				3:PRTIN1U,		4:SWCLK		7:
	;
	PB->MR1 = 0
	| (0<<28)				// 0:PB7,			1:			2:				3:OVIN0U,			4:CLKO		7:
	| (0<<24)				// 0:PB6,			1:			2:				3:PRTIN0U,		4:		7:
	| (0<<20)				// 0:PB5,			1:			2:T5IO			3:MP0WL,			4:		7:
	| (0<<16)				// 0:PB4,			1:			2:T4IO			3:MP0WH,		4:		7:
	| (0<<12)				// 0:PB3,			1:			2:				3:MP0VL,			4:		7:
	| (0<<8)				// 0:PB2,			1:			2:				3:MP0VH,			4:		7:
	| (0<<4)				// 0:PB1,			1:			2:				3:MP0UL,			4:		7:
	| (0<<0)				// 0:PB0,			1:			2:				3:MP0UH,			4:		7:
	;
	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO)
	PB->CR = 0
	| (0<<30)   // P15 		0:Push-pull output, 1:Open-drain output, 2,3:Input
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	// Pullup/down Setting
	PB->PRCR = 0
	| (0<<30)   // P15 		0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	
	//------------------------------------------------------------
	//	PORT - C
	//------------------------------------------------------------
	// PinMux Setting
	PC->MR2 = 0
	| (3<<28)				// 0:PC15,		1:			2:				3:TXD0,			4:		7:
	| (3<<24)				// 0:PC14,		1:			2:				3:RXD0,			4:		7:
	| (7<<20)				// 0:PC13,		1:			2:				3:				4:		7:XOUT
	| (7<<16)				// 0:PC12,		1:			2:				3:				4:		7:XIN
	| (3<<12)				// 0:PC11,		1:			2:T6IO,			3:BOOT,			4:		7:
	| (3<<8)				// 0:PC10,		1:			2:				3:nRESET,		4:		7:
	| (3<<4)				// 0:PC9,			1:			2:T5IO,			3:CLKO,			4:		7:
	| (0<<0)				// 0:PC8,			1:SDA0,		2:T4IO,			3:QEI1_IDX		4:		7:
	;

	PC->MR1 = 0
	| (0<<28)				// 0:PC7,			1:SCL0,		2:T3IO,			3:QEI1_B			4:		7:
	| (0<<24)				// 0:PC6,			1:TXD1,		2:T2IO,			3:QEI1_A			4:		7:
	| (0<<20)				// 0:PC5,			1:RXD1,		2:T1IO,			3:QEI1_UPDN		4:		7:
	| (0<<16)				// 0:PC4,			1:			2:T0IO,			3:				4:		7:
	| (0<<12)				// 0:PC3,			1:			2:				3:				4:		7:
	| (0<<8)				// 0:PC2,			1:			2:				3:				4:		7:
	| (3<<4)				// 0:PC1,			1:TXD0,		2:				3:SWDIO,			4:		7:
	| (3<<0)				// 0:PC0,			1:RXD0,		2:				3:SWCLK,			4:BOOT	7:
	;

	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO) 
	PC->CR = 0
	| (0<<30)   // P15 		0:Push-pull output, 1:Open-drain output, 2,3:Input
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	// Pullup/down Setting
	PC->PRCR = 0
	| (0<<30)   // P15 		0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	PD->MR1 = 0
	| (0<<12)				// 0:PD3,		1:MISO0,		2:		3:SDA1,		4:		7:AN19
	| (0<<8)				// 0:PD2,		1:MOSI0,		2:		3:SCL1,		4:		7:AN18
	| (0<<4)				// 0:PD1,		1:SCK0,		2:		3:T7IO,		4:		7:AN17
	| (0<<0)				// 0:PD0,		1:SS0,		2:		3:T6IO,		4:		7:AN16
	;
	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO) 
	PD->CR = 0
	| (0<<6)   // P3			0:Push-pull output, 1:Open-drain output, 2,3:Input
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	// Pullup/down Setting
	PD->PRCR = 0	
	| (0<<6)   // P3			0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	//------------------------------------------------------------
	//	PORT - F
	//------------------------------------------------------------
	
	// PinMux Setting
	
	PF->MR1 = 0
	| (0<<28)				// 0:PF7,			1:			2:			3:OVINEW,	4:		7:
	| (0<<24)				// 0:PF6,			1:			2:			3:PRTINEW,	4:		7:
	| (0<<16)				// 0:PF4,			1:			2:			3:,			4:		7:
	| (0<<12)				// 0:PF3,			1:			2:			3:,			4:		7:
	| (0<<8)				// 0:PF2,			1:			2:			3:,			4:		7:
	| (0<<4)				// 0:PF1,			1:SDA1		2:			3:,			4:		7:
	| (0<<0)				// 0:PF0,			1:SCL1		2:			3:,			4:		7:
	;
	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO)
	PF->CR = 0
	| (0<<14)   // P7 		0:Push-pull output, 1:Open-drain output, 2,3:Input
	| (0<<12)   // P6
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	
	// Pullup/down Setting
	PF->PRCR = 0
	| (0<<14)   // P7		0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	| (0<<12)   // P6
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
	;
	

		
	//------------------------------------------------------------
	//	PORT - G
	//------------------------------------------------------------
	
	// PinMux Setting
	PG->MR2 = 0
	| (0<<8)				// 0:PG10,	1:			2:T7IO	3:				4:		7:
	;
	
	//Input, Output, Open Drain (only used when PinMux Setting as GPIO)
	PG->CR = 0
	| (0<<20)   // P10 		0:Push-pull output, 1:Open-drain output, 2,3:Input
	;
	
	// Pullup/down Setting
	PG->PRCR = 0
	| (0<<20)   // P10 		0,1:Disable pull-up/down resistor, 2:Enable pull-up resistor, 3:Enable Pull-down resistor
	;
	
	// Disable Writing Permittion of ALL PCU register.
	PORT_ACCESS_DIS();
	
}
